/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.util.*;
import socle.lexicon.*;
import socle.conc.*;
import socle.map.*;
import socle.trame.*;
import socle.cadre.*;
import socle.selection.*;
import socle.partition.*;
import socle.preferences.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.nio.charset.Charset;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.*;
import javax.swing.undo.*;
import javax.swing.event.*;

import org.annolab.tt4j.*;
/*-----------------------------------------------------------------------*/

/** Programme de textometrie.
*/
public class Textometre extends JFrame implements PreferencesListener {
		private LexModel               lexicon;
		private TrameModel             trame;
		private CadreModel             cadre;
		private CadreController        cadreController;
		private TrameController        trameController;
		private LexController          lexController;
		private ConcView               frameConcordancier;
		private TrameCadreView         frameTrameCadre;
		private MapView                frameSectionsMap;
		private UndoAction             undoAction;
		private RedoAction             redoAction;
		private UndoManager            undoManager;
		private PreferencesController  preferencesController;
		private HashMap<String,String> listMenuItems;
		private JButton                openButton, importButton, saveButton, colorsButton, findButton;

		public static PreferencesResourceBundle    LABELS;
		public static UndoableEditSupport          UNDO_SUPPORT;
		public static String TEXTOMETRE_NAMESPACE = "http://www.textometrie.fr/schemas/";
		public static String TEXTOMETRE_EXTENSION = "txm";

	public static void main(String [] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Textometre thisClass = new Textometre();
				thisClass.setVisible(true);
			}
		});
	}
	public static String getProperty(String property) {
		Properties configFile = new Properties();
		try {
			configFile.loadFromXML(socle.Textometre.class.getResourceAsStream("/resources/Textometre.ini"));
			return configFile.getProperty(property);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	public static void cleanUndo() {
		UndoableEditListener[] listenerList = UNDO_SUPPORT.getUndoableEditListeners();
		for(int i=0; i<listenerList.length; i++) {
			if (listenerList[i] instanceof UndoAdapter) {
				UndoAdapter adapter = (UndoAdapter)listenerList[i];
				adapter.cleanAll();
			}
		}
	}

	/** Cree la fenetre principale.
	*/
	public Textometre() {
		super();

		setLayout(new BorderLayout() );
		addWindowListener (new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				quit();
			}
		});
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		PreferencesModel preferencesModel = new PreferencesModel();
		LABELS = new PreferencesResourceBundle(preferencesModel.getLocale());
		this.preferencesController = new PreferencesController(preferencesModel);

		this.undoAction = new UndoAction();
		this.redoAction = new RedoAction();
		this.undoManager = new UndoManager();
		UNDO_SUPPORT = new UndoableEditSupport();
		UNDO_SUPPORT.addUndoableEditListener(new UndoAdapter());

		this.listMenuItems = new HashMap<String,String>();
		listMenuItems.put("menu",             "menu.file/menu.edit/menu.preferences/menu.actions/menu.windows/menu.language/menu.help");
		listMenuItems.put("menu.file",        "file.open/file.save/-/file.import/file.importURL/-/file.openCadre/file.saveCadre/-/file.quit");
		listMenuItems.put("menu.edit",        "edit.undo/edit.redo/-/edit.find");
		listMenuItems.put("menu.preferences", "preferences.display/preferences.collators/preferences.tokenizer");
		listMenuItems.put("menu.actions",     "actions.newSelection/actions.newPartition/actions.lemmatizer/actions.typage");
		listMenuItems.put("menu.windows",     "windows.concordances/windows.cadreTrame/windows.sectionsMap");
		listMenuItems.put("menu.language",    "language.en/language.fr");
		listMenuItems.put("menu.help",        "help.about");
		listMenuItems.put("file.import",      "file.importTXT/file.importXML/-/file.importODT/file.importRTF/file.importHTML");

		setTitle("Textometre");
		setBackground(Color.white);
		getContentPane().setLayout(new BorderLayout());

		JMenuBar mb = createMenubar(listMenuItems);
		JDesktopPane desktop = new JDesktopPane();
		//desktop.setLayout();
		setJMenuBar(mb);

		getContentPane().add(desktop, BorderLayout.CENTER);
        Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
		setSize(screenSize);

		/* La toolbar
		*/
		JToolBar toolBar = new JToolBar("Toolbar");
		openButton = new JButton(new ImageIcon(getClass().getResource("/icons/fileopen.png")));
		openButton.setToolTipText("file.open");
		openButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				open_data();
			}
		});
		importButton = new JButton(new ImageIcon(getClass().getResource("/icons/fileimport.png")));
		importButton.setToolTipText("file.importTXT");
		importButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				importTXT();
			}
		});
		saveButton = new JButton(new ImageIcon(getClass().getResource("/icons/filesave.png")));
		saveButton.setToolTipText("file.save");
		saveButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				save_data();
			}
		});

		colorsButton = new JButton(new ImageIcon(getClass().getResource("/icons/colorize.png")));
		colorsButton.setToolTipText("preferences.bgColor");
		colorsButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				displayOptions();
			}
		});
		findButton = new JButton(new ImageIcon(getClass().getResource("/icons/find.png")));
		findButton.setToolTipText("edit.find");
		findButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				find();
			}
		});
    	toolBar.add(openButton);
    	toolBar.add(importButton);
    	toolBar.add(saveButton);
    	toolBar.addSeparator();
    	toolBar.add(findButton);
    	toolBar.addSeparator();
    	toolBar.add(colorsButton);
		getContentPane().add(toolBar, BorderLayout.PAGE_START);

		this.lexicon = new LexModel();
		this.lexController = new LexController(lexicon);
		lexController.addPreferencesListener(preferencesModel);

		this.trame = new TrameModel(lexController);
		this.trameController = new TrameController(trame);
		lexController.addTrameListener(trame);

		this.cadre = new CadreModel();
		trameController.addCadreListener(cadre);
		trameController.addLexListener(lexicon);
		trameController.addPreferencesListener(preferencesModel);
		this.cadreController = new CadreController(cadre);

		this.frameTrameCadre  = new TrameCadreView(trame, cadre, lexicon, trameController, cadreController, lexController);
		desktop.add(frameTrameCadre);
		try {
			frameTrameCadre.setVisible(true);
			frameTrameCadre.setSelected(true);
		} catch (java.beans.PropertyVetoException err) {
			System.err.println(err.getMessage());
		}

		this.frameConcordancier  = new ConcView(frameTrameCadre);
		trame.addTrameListener(frameConcordancier);
		desktop.add(frameConcordancier);
		try {
			frameConcordancier.setVisible(true);
			frameConcordancier.setSelected(true);
		} catch (java.beans.PropertyVetoException err) {
			System.err.println(err.getMessage());
		}

		this.frameSectionsMap = new MapView(trame, cadre);
		cadre.addCadreListener(frameSectionsMap);
		frameSectionsMap.setBounds(0,  0, (int)screenSize.getWidth() * 65 / 100, (int)screenSize.getHeight() - 120);
		desktop.add(frameSectionsMap);

		preferencesModel.addPreferencesListener(trame);
		preferencesModel.addPreferencesListener(frameSectionsMap);
		preferencesModel.addPreferencesListener(frameConcordancier);
		preferencesModel.addPreferencesListener(this);
		preferencesModel.setLocale(preferencesModel.getLocale());
	}

	private void quit() {
		dispose();
		System.exit(0);
	}
	private JMenuBar createMenubar(HashMap<String,String> hash) {
		JMenuItem mi;
		JMenuBar mb = new JMenuBar();

		String liste = hash.get("menu");
		StringTokenizer menuKeys = new StringTokenizer(liste, "/");
		while (menuKeys.hasMoreTokens()) {
			String name = menuKeys.nextToken();
			JMenu m = createMenu(name, hash);
			if (m != null) {
				mb.add(m);
			}
		}
		return mb;
	}
	private JMenu createMenu(String key, HashMap<String,String> hash) {
		final JMenu menu = new JMenu(key);
		String s = hash.get(key);
		StringTokenizer menuItems = new StringTokenizer(s, "/");
		while (menuItems.hasMoreTokens()) {
			String name = menuItems.nextToken();
			if (name.equals("-")) {
				menu.addSeparator();
			} else if (name.equals("file.import")) {
				JMenu subMenu = new JMenu(name);
				String su = hash.get(name);
				StringTokenizer subMenuItems = new StringTokenizer(su, "/");
				while (subMenuItems.hasMoreTokens()) {
					String label = subMenuItems.nextToken();
					if (label.equals("-")) {
						subMenu.addSeparator();
					} else if (label.equals("file.importTXT")) {
						final JMenuItem smi = new JMenuItem(label, new ImageIcon(getClass().getResource("/icons/fileimport.png")));
						smi.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								importTXT();
							}
						});
						subMenu.add(smi);
					} else if (label.equals("file.importXML")) {
						final JMenuItem smi = new JMenuItem(label);
						smi.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								importXML();
							}
						});
						subMenu.add(smi);
					} else if (label.equals("file.importODT")) {
						final JMenuItem smi = new JMenuItem(label);
						smi.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								importODT();
							}
						});
						subMenu.add(smi);
					} else if (label.equals("file.importRTF")) {
						final JMenuItem smi = new JMenuItem(label);
						smi.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								importRTF();
							}
						});
						subMenu.add(smi);
					} else if (label.equals("file.importHTML")) {
						final JMenuItem smi = new JMenuItem(label);
						smi.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent e) {
								importHTM();
							}
						});
						subMenu.add(smi);
					}
				}
				menu.add(subMenu);
			} else if (name.equals("file.openCadre")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						frameTrameCadre.openCadre();
					}
				});
				menu.add(mi);
			} else if (name.equals("file.importURL")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						importURL();
					}
				});
				menu.add(mi);
			} else if (name.equals("file.saveCadre")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						frameTrameCadre.saveCadre();
					}
				});
				menu.add(mi);
			} else if (name.equals("file.save")) {
				final JMenuItem mi = new JMenuItem(name, new ImageIcon(getClass().getResource("/icons/filesave.png")));
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						save_data();
					}
				});
				menu.add(mi);
			} else if (name.equals("file.open")) {
				final JMenuItem mi = new JMenuItem(name, new ImageIcon(getClass().getResource("/icons/fileopen.png")));
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						open_data();
					}
				});
				menu.add(mi);
			} else if (name.equals("file.quit")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						quit();
					}
				});
				menu.add(mi);
			} else if (name.equals("preferences.display")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						displayOptions();
					}
				});
				menu.add(mi);
			} else if (name.equals("edit.find")) {
				final JMenuItem mi = new JMenuItem(name, new ImageIcon(getClass().getResource("/icons/find.png")));
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						find();
					}
				});
				menu.add(mi);
			} else if (name.equals("edit.undo")) {
				final JMenuItem mi = new JMenuItem(undoAction);
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
				menu.add(mi);
			} else if (name.equals("edit.redo")) {
				final JMenuItem mi = new JMenuItem(redoAction);
				mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
				menu.add(mi);
			} else if (name.equals("preferences.collators")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						preferencesController.displayCollatorView();
					}
				});
				menu.add(mi);
			} else if (name.equals("preferences.tokenizer")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						chooseTokenizer();
					}
				});
				menu.add(mi);
			} else if (name.equals("windows.concordances")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
					try {
						frameConcordancier.setVisible(true);
						frameConcordancier.setSelected(true);
					} catch (java.beans.PropertyVetoException err) {
						System.err.println(err.getMessage());
					}
					}
				});
				menu.add(mi);
			} else if (name.equals("windows.cadreTrame")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
					try {
						frameTrameCadre.setVisible(true);
						frameTrameCadre.setSelected(true);
					} catch (java.beans.PropertyVetoException err) {
						System.err.println(err.getMessage());
					}
					}
				});
				menu.add(mi);
			} else if (name.equals("windows.sectionsMap")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						try {
							frameSectionsMap.setVisible(true);
							frameSectionsMap.setSelected(true);
						} catch (java.beans.PropertyVetoException err) {
							System.err.println(err.getMessage());
						}
					}
				});
				menu.add(mi);
			} else if (name.equals("actions.newSelection")) {
				Action actionSelection = new SetSelectionAction(name);
				final JMenuItem mi = new JMenuItem(actionSelection);
				menu.add(mi);
			} else if (name.equals("actions.newPartition")) {
				Action actionPartition = new SetPartitionAction(name);
				final JMenuItem mi = new JMenuItem(actionPartition);
				menu.add(mi);
			} else if (name.equals("actions.lemmatizer")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						lemmatize();
					}
				});
				menu.add(mi);
			} else if (name.equals("actions.typage")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						typage();
					}
				});
				menu.add(mi);
			} else if (name.equals("language.fr")) {
				final JCheckBoxMenuItem mi = new JCheckBoxMenuItem(name, preferencesController.getLocale().getLanguage().equals("fr"));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						preferencesController.setLocale(Locale.FRENCH);
						for (int i=0; i<menu.getItemCount(); i++) {
							JMenuItem mymi = menu.getItem(i);
							mymi.setSelected(mymi==mi);
						}
					}
				});
				menu.add(mi);
			} else if (name.equals("language.en")) {
				final JCheckBoxMenuItem mi = new JCheckBoxMenuItem(name, !preferencesController.getLocale().getLanguage().equals("fr"));
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						preferencesController.setLocale(Locale.ENGLISH);
						for (int i=0; i<menu.getItemCount(); i++) {
							JMenuItem mymi = menu.getItem(i);
							mymi.setSelected(mymi==mi);
						}
					}
				});
				menu.add(mi);
			} else if (name.equals("help.about")) {
				final JMenuItem mi = new JMenuItem(name);
				mi.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e) {
						about();
					}
				});
				menu.add(mi);
			}
		}
		return menu;
	}
    private String[] getTopLevelMenu() {
		return getMenu("menu");
	}
    private String[] getMenu(String menu) {
		String listMenu = (String)listMenuItems.get(menu);
		return listMenu.split("/");
	}

	/********************************************************************
	**   Affiche les credits et la licence GPL
	*********************************************************************/
	private void about() {
		StringBuffer content = new StringBuffer();
		BufferedReader reader = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/resources/COPYING.txt")));
		String line;
		try {
			while ((line=reader.readLine())!=null) {
				content.append(line).append("\n");
			}
			reader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(new JLabel("Copyright (c) 2009 Michel Jacobson"), BorderLayout.NORTH);
		JTextArea textArea = new JTextArea(content.toString());
		panel.add(new JScrollPane(textArea), BorderLayout.CENTER);
		panel.setPreferredSize(new Dimension(550, 500));
		JOptionPane.showConfirmDialog(this, panel, LABELS.getLocaliseString("label.about"), JOptionPane.DEFAULT_OPTION, JOptionPane.PLAIN_MESSAGE);
	}
	/********************************************************************
	**   Recupere l'extension d'un fichier (a placer ailleur)
	*********************************************************************/
	public static String getExtension(File f) {
		String ext = null;
		String s = f.getName();
		int i = s.lastIndexOf('.');
		if (i > 0 &&  i < s.length() - 1) {
			ext = s.substring(i+1).toLowerCase();
		}
		return ext;
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier texte au format HTML
	*********************************************************************/
	private void importURL() {
		String response = JOptionPane.showInputDialog(null, LABELS.getLocaliseString("file.importURL"), "URL: ", JOptionPane.QUESTION_MESSAGE);
		if (response != null) {
			frameTrameCadre.openUrl(response);
		}
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier texte au format XML
	*********************************************************************/
	private void importXML() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("xml");
		filter.setDescription("XML Files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open XML file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			frameTrameCadre.openFile(fic, "xml");
		}
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier texte au format ODT
	*********************************************************************/
	private void importODT() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("odt");
		filter.setDescription("ODT Files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open ODT file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			frameTrameCadre.openFile(fic, "odt");
		}
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier texte au format RTF
	*********************************************************************/
	private void importRTF() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("rtf");
		filter.setDescription("RTF Files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open RTF file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			frameTrameCadre.openFile(fic, "rtf");
		}
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier texte au format HTML
	*********************************************************************/
	private void importHTM() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("htm");
		filter.addExtension("html");
		filter.setDescription("HTML (version 3.2) Files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open HTML file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			frameTrameCadre.openFile(fic, "htm");
		}
	}
	/********************************************************************
	**   Ouvrir un nouveau fichier au format texte-seul
	*********************************************************************/
	private void importTXT() {
		SortedMap charsetList = Charset.availableCharsets();
		Object[] possibleValues = charsetList.values().toArray();
		JPanel custom = new JPanel();
		final JComboBox charsets = new JComboBox(possibleValues);
		charsets.setSelectedItem(Charset.defaultCharset());
		final JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("txt");
		filter.setDescription("Text Files");
		chooser.setFileFilter(filter);

		final JDialog dialog = new JDialog();
		dialog.setModal(true);
		dialog.setTitle("Open");
		chooser.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dialog.dispose();
				if (!e.getActionCommand().equals(JFileChooser.CANCEL_SELECTION) && chooser.getSelectedFile() != null) {
					File fic = chooser.getSelectedFile();
					Charset charset = (Charset)charsets.getSelectedItem();
					frameTrameCadre.openFile(fic, charset);
				}
			}
		});
		custom.add(chooser);
		custom.add(charsets);
		dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		dialog.getContentPane().add(custom, BorderLayout.CENTER);
		dialog.pack();
		dialog.setVisible(true);
	}
	/********************************************************************
	**   Importe les donnees d'un fichier xml
	*********************************************************************/
	private void open_data() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension(TEXTOMETRE_EXTENSION);
		filter.setDescription("Textometre Files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open Textometre file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			frameTrameCadre.import_data(fic);
		}
	}
	/********************************************************************
	**   Exporte les donnees dans un fichier au format XML
	*********************************************************************/
	private void save_data() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension(TEXTOMETRE_EXTENSION);
		filter.setDescription("Textometre Files");
		chooser.setFileFilter(filter);
		chooser.setSelectedFile(new File("untitled.txm"));
		if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			int answer = JOptionPane.OK_OPTION;
			if (fic.exists()) {
				answer = JOptionPane.showConfirmDialog(null, fic.getAbsolutePath()+" "+Textometre.LABELS.getLocaliseString("message.fic.exists"), JOptionPane.MESSAGE_PROPERTY, JOptionPane.YES_NO_OPTION);
			}
			if (answer == JOptionPane.OK_OPTION) {
				try {
					frameTrameCadre.export_data(fic);
				} catch (Exception err) {
					System.err.println(err.getMessage());
				}
			}
		}
	}
	private void displayOptions() {
		preferencesController.displayOptions(trameController.getPropertyNames(), trameController.getFacets());
	}
	private void chooseTokenizer() {
		preferencesController.displayTokenizerView();
	}
	private void find() {
		trameController.find();
	}
	private void lemmatize() {
		JFileChooser chooser = new JFileChooser(Textometre.getProperty("TREETAGGER_DIR_LIB"));
		myFileFilter filter = new myFileFilter();
		filter.addExtension("par");
		filter.setDescription("TreeTagger parameter files");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Open TreeTagger parameter file");
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();

			final Tnum tnum_pos = new Tnum();
			final Tnum tnum_lemma = new Tnum();
			final Tnum tnum = trameController.getTnum(Tnum.GRAPH);
			for (int i=0; i< tnum.size(); i++) {
				String val = tnum.getString(i);
				if ( (!val.equals("\n")) && (!val.equals(" ")) ) {
					val = "unknown";
				}
				tnum_pos.add(val);
				tnum_lemma.add(val);
			}
			TreeTaggerWrapper<Integer> tt = new TreeTaggerWrapper<Integer>();
			try {
				//on remplis les tnums
				tt.setModel(fic.toString()+":iso-8859-1");
				tt.setAdapter(new TokenAdapter<Integer>() {
					public String getText(Integer o) {
						int i = o.intValue();
						return tnum.getString(i);
					}
				});
				tt.setHandler(new TokenHandler<Integer>() {
					public void token(Integer token, String pos, String lemma) {
						if (pos == null) pos = "null";
						if (lemma == null) lemma = "null";
						int index = token.intValue();
						tnum_pos.set(index, pos);
						tnum_lemma.set(index, lemma);
					}
				});
				ArrayList<Integer> liste = new ArrayList<Integer>();
				for (int i=0; i<tnum.size(); i++) {
					liste.add(new Integer(i));
				}
				tt.process(liste);
				trameController.setTnum("POS", tnum_pos);
				trameController.setTnum("LEMMA", tnum_lemma);
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				tt.destroy();
			}
		}
	}
	private void typage() {
		Object[] objects = preferencesController.getTypeDefinition();
		int firstTnum = -1;
		for (int i=0; ((i<objects.length) && (firstTnum == -1)); i++) {
			if (objects[i] instanceof String) {
				firstTnum = i;
			}
		}
		if (firstTnum != -1) {
			Tnum tnum_type  = trameController.getTnum(objects[0].toString());
			for (int i=firstTnum+1; i<objects.length; i++) {
				if (objects[i] instanceof Litteral) {
					tnum_type = tnum_type.croise(objects[i].toString());
				} else {
					tnum_type = tnum_type.croise(trameController.getTnum(objects[i].toString()));
				}
			}
			trameController.setTnum("TYPE", tnum_type);
		}
	}

	public void preferencesChanged(PreferencesChangedEvent event) {
		if(event instanceof PreferencesChangedLocaleEvent) {
			LABELS = ((PreferencesChangedLocaleEvent)event).getLabels();
			String[] topLevelMenus = getTopLevelMenu();
			JMenuBar menuBar = getJMenuBar();
			for (int i=0; i<menuBar.getMenuCount(); i++) {
				JMenu menu = menuBar.getMenu(i);
				menu.setText(LABELS.getLocaliseString(topLevelMenus[i]));
				String[] itemMenus = getMenu(topLevelMenus[i]);
				for (int j=0; j<menu.getItemCount(); j++) {
					JMenuItem menuItem = menu.getItem(j);
					if (! itemMenus[j].equals("-")) {
						menuItem.setText(LABELS.getLocaliseString(itemMenus[j]));
					}
					if (itemMenus[j].equals("file.import")) {
						JMenu submenu = (JMenu)menuItem;
						String[] itemsubMenus = getMenu("file.import");
						for (int k=0; k<submenu.getItemCount(); k++) {
							JMenuItem submenuItem = submenu.getItem(k);
							if (! itemsubMenus[k].equals("-")) {
								submenuItem.setText(LABELS.getLocaliseString(itemsubMenus[k]));
							}
						}
					}
				}
			}
			openButton.setToolTipText(LABELS.getLocaliseString("file.open"));
			importButton.setToolTipText(LABELS.getLocaliseString("file.importTXT"));
			saveButton.setToolTipText(LABELS.getLocaliseString("file.save"));
			colorsButton.setToolTipText(LABELS.getLocaliseString("preferences.bgColor"));
			findButton.setToolTipText(LABELS.getLocaliseString("edit.find"));
		}
	}

	/********************************************************************
	**   Action Undo
	*********************************************************************/
    class UndoAction extends AbstractAction {
		public UndoAction() {
			super(LABELS.getLocaliseString("edit.undo"));
			setEnabled(false);
		}
		public void actionPerformed(ActionEvent e) {
			try {
				undoManager.undo();
			} catch (CannotUndoException ex) {
				System.err.println("Unable to undo: " + ex);
				ex.printStackTrace();
			}
			update();
			redoAction.update();
		}
		protected void update() {
			if(undoManager.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, undoManager.getUndoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, LABELS.getLocaliseString("edit.undo"));
			}
		}
    }

	/********************************************************************
	**   Action Redo
	*********************************************************************/
    class RedoAction extends AbstractAction {
		public RedoAction() {
			super(LABELS.getLocaliseString("edit.redo"));
			setEnabled(false);
		}
		public void actionPerformed(ActionEvent e) {
			try {
				undoManager.redo();
			} catch (CannotRedoException ex) {
				System.err.println("Unable to redo: " + ex);
				ex.printStackTrace();
			}
			update();
			undoAction.update();
		}
		protected void update() {
			if(undoManager.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, undoManager.getRedoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, LABELS.getLocaliseString("edit.redo"));
			}
		}
    }
	/********************************************************************
	**   UndoAdapter
	*********************************************************************/
	private class UndoAdapter implements UndoableEditListener {
		public void undoableEditHappened (UndoableEditEvent evt) {
			UndoableEdit edit = evt.getEdit();
			undoManager.addEdit( edit );
			undoAction.update();
			redoAction.update();
		}
		public void cleanAll() {
			undoManager.discardAllEdits();
		}
	}
	/********************************************************************
	**   Action d'ajout ou de modification de selection dans le cadre
	*********************************************************************/
	public class SetSelectionAction extends AbstractAction {
		public SetSelectionAction(String message) {
			super(message);
		}
		public void addContent(AbstractSelection sel) {
			cadreController.addSelection(sel);
		}
		public void removeContent(AbstractSelection sel) {
			cadreController.delSelection(sel);
		}
		public void actionPerformed(ActionEvent e) {
			AbstractSelection sel = frameTrameCadre.getNewSelection(trameController.getSelectionSelected());
			if (sel != null) {
				if (cadreController.containsSelection(sel.getTitle())) {
					AbstractSelection sel_old = cadreController.getSelection(sel.getTitle());
					UndoableEdit edit = new SelectionEdit(this, sel_old, sel);
					UNDO_SUPPORT.postEdit(edit);
				} else {
					UndoableEdit edit = new SelectionEdit(this, null, sel);
					UNDO_SUPPORT.postEdit(edit);
				}
			}
		}
	}
	/********************************************************************
	**   Action de modification de la partition dans le cadre
	*********************************************************************/
	public class SetPartitionAction extends AbstractAction {
		public SetPartitionAction(String message) {
			super(message);
		}
		public void addContent(Partition partition) {
			cadreController.addPartition(partition);
		}
		public void removeContent(Partition partition) {
			cadreController.delPartition(partition);
		}
		public void actionPerformed(ActionEvent e) {
			Partition part = frameTrameCadre.getNewPartition();
			if (part != null) {
				if (cadreController.containsPartition(part.getTitle())) {
					Partition part_old = cadreController.getPartition(part.getTitle());
					UndoableEdit edit = new PartitionEdit(this, part_old, part);
					UNDO_SUPPORT.postEdit(edit);
				} else {
					UndoableEdit edit = new PartitionEdit(this, null, part);
					UNDO_SUPPORT.postEdit(edit);
				}
			}
		}
	}
	/********************************************************************
	**   Action reversible d'ajout / remplacement de selection
	*********************************************************************/
	public class SelectionEdit extends AbstractUndoableEdit {

			private SetSelectionAction action;
			private AbstractSelection  sel_old;
			private AbstractSelection  sel_new;

		public SelectionEdit(Object obj, AbstractSelection sel_old, AbstractSelection sel_new) {
			this.action = (SetSelectionAction)obj;
			this.sel_new = sel_new;
			this.sel_old = sel_old;
			if (sel_old!=null) action.removeContent(sel_old);
			action.addContent(sel_new);
		}
		public void undo() throws CannotUndoException {
			super.undo();
			action.removeContent(sel_new);
			if (sel_old!=null) action.addContent(sel_old);
		}
		public void redo() throws CannotRedoException {
			super.redo();
			if (sel_old!=null) action.removeContent(sel_old);
			action.addContent(sel_new);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			if (sel_old!=null) return LABELS.getLocaliseString("action.replace.selection");
			return LABELS.getLocaliseString("action.append.selection");
		}
	}
	/********************************************************************
	**   Action reversible d'ajout / remplacement de partition
	*********************************************************************/
	public class PartitionEdit extends AbstractUndoableEdit {

			private SetPartitionAction action;
			private Partition          partition_old;
			private Partition          partition_new;

		public PartitionEdit(Object obj, Partition partition_old, Partition partition_new) {
			this.action = (SetPartitionAction)obj;
			this.partition_new = partition_new;
			this.partition_old = partition_old;
			if (partition_old!=null) action.removeContent(partition_old);
			action.addContent(partition_new);
		}
		public void undo() throws CannotUndoException {
			action.removeContent(partition_new);
			if (partition_old!=null) action.addContent(partition_old);
		}
		public void redo() throws CannotRedoException {
			if (partition_old!=null) action.removeContent(partition_old);
			action.addContent(partition_new);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			if (partition_old!=null) return LABELS.getLocaliseString("action.replace.partition");
			return LABELS.getLocaliseString("action.append.partition");
		}

	}
	/********************************************************************
	**   Filtre pour fichiers
	*********************************************************************/
	private class myFileFilter extends FileFilter {
		private String description = "All Files";
		private ArrayList<String> extensions = new ArrayList<String>();

		//Accepte les repertoires et les fichiers avec les extensions definies.
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			String extension = Textometre.getExtension(f);
			if (extension != null) {
				if (extensions.contains(extension)) {
					return true;
				} else {
					return false;
				}
			}
			return false;
		}

		//Description du filtre
		public String getDescription() {
			return description;
		}
		public void setDescription(String description) {
			this.description = description;
		}
		public void addExtension(String extension) {
			extensions.add(extension);
		}
	}
};
