/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import socle.util.*;
import socle.trame.*;
import socle.cadre.*;
import socle.partition.*;
import socle.selection.*;
import socle.reader.*;
import socle.tokenizer.*;
import socle.preferences.*;
import socle.lexicon.*;
import socle.lexicon.article.*;
import socle.conc.*;

import java.util.*;
import java.io.*;
import java.net.*;
import java.nio.charset.Charset;
import java.text.Collator;
import java.awt.*;
import javax.swing.*;
import javax.swing.undo.*;

import java.awt.event.*;
/*-----------------------------------------------------------------------*/

/** Fenetre affichant la trame textuelle et son cadre de navigation.
*/
public class TrameCadreView extends JInternalFrame {

		private LexModel              lexicon;
		private CadreModel            cadre;
		private CadreController       cadreController;
		private TrameModel            trame;
		private TrameController       trameController;
		private LexController         lexController;

	/**
	 * Cree une fenetre dans laquelle s'affiche le cadre et la trame.
	 **/
	public TrameCadreView(TrameModel trame, CadreModel cadre, LexModel lexicon, TrameController trameController, CadreController cadreController, LexController lexController) {
		super(Textometre.LABELS.getLocaliseString("windows.cadreTrame"), true);

		Dimension screenSize = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
		setBounds(0,  0, (int)screenSize.getWidth() * 80 / 100, (int)screenSize.getHeight() - 120);

		this.lexicon = lexicon;
		this.trame = trame;
		this.cadre = cadre;

		JPanel panel_lexique = new JPanel(new BorderLayout());
		JPanel panel_trame = new JPanel(new BorderLayout());
		JPanel panel_cadre = new JPanel(new BorderLayout());
		JSplitPane splitPanelTrameEditor = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel_lexique, panel_trame);
		JSplitPane splitPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel_cadre, splitPanelTrameEditor);

		this.trameController = trameController;
		trameController.displayView(panel_trame);
		this.lexController = lexController;
		lexController.displayView(panel_lexique);
		this.cadreController = cadreController;
		cadreController.displayView(panel_cadre);

		setContentPane(splitPanel);
		//pack();
	}
	public CadreController getCadreController() {
		return cadreController;
	}

	/**
	 * Retourne la trame textuelle
	 **/
	public TrameModel getTrame() {
		return trame;
	}
	/**
	 * Chargement d'un cadre .a partir d'un fichier
	 **/
	public void openCadre() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			cadre.loadFromFile(fic);
		}
	}
	/**
	 * Sauvegarde du cadre .dans un fichier
	 **/
	public void saveCadre() {
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			File fic = chooser.getSelectedFile();
			int answer = JOptionPane.OK_OPTION;
			if (fic.exists()) {
				answer = JOptionPane.showConfirmDialog(null, fic.getAbsolutePath()+" "+Textometre.LABELS.getLocaliseString("message.fic.exists"), JOptionPane.MESSAGE_PROPERTY, JOptionPane.YES_NO_OPTION);
			}
			if (answer == JOptionPane.OK_OPTION) {
				cadre.writeToFile(fic);
			}
		}
	}

	/**
	 * Calcule une partition sur la trame a l'aide de l'item separateur.
	 **/
	public Partition getNewPartition() {
		DialogPartition dial = new DialogPartition();
		dial.setVisible(true);
		if (dial.isValidate()) {
			String separatorItem = dial.getSeparatorItem();
			separatorItem = separatorItem.replaceAll("\\\\n", "\n");
			separatorItem = separatorItem.replaceAll("\\\\r", "\r");
			separatorItem = separatorItem.replaceAll("\\\\t", "\t");
			String title = dial.getPartitionName();
			if ((title != null) && (!title.equals("")) && (separatorItem != null)  && (!separatorItem.equals(""))) {
				if (cadreController.containsPartition(title)) {
					int response = JOptionPane.showConfirmDialog(null, Textometre.LABELS.getLocaliseString("message.partition.modify")+": '"+title+"'", JOptionPane.MESSAGE_PROPERTY, JOptionPane.YES_NO_OPTION);
					if (response != JOptionPane.YES_OPTION) {
						return null;
					}
				}
				//il faut creer un nouvel objet Partition a chaque fois (pour le undo...)
				return computePartition(separatorItem, title);
			}
		}
		return null;
	}
	private Partition computePartition(String separator, String title) {
		Partition part = new Partition(title);
		ArrayList<ZoneTrame> list = new ArrayList<ZoneTrame>();
		try {
			Article art = trame.getArticle(separator);
			if (art != null) {
				int numApp = art.getNumap();
				ZoneTrame zt;
				int start = 0;
				for (int pos=0; pos<trame.size(); pos++) {
					int n = trame.getNumap(pos);
					if (n == numApp) {
						if (pos > start) {
							zt = new ZoneTrame(start, pos);
							list.add(zt);
						}
						start = pos+1;
					}
				}
				if (trame.size()-1 > start) {
					zt = new ZoneTrame(start, trame.size()-1);
					list.add(zt);
				}
			}
		} catch (Exception e) {}
		part.setPartition(list);
		return part;
	}

	/**
	 * Charge une nouvelle trame et un nouveau cadre  partir d'une url.
	 * @param str     URL du contenu textuel  lire
	 **/
	public void openUrl(String str) {
		try {
			URL url = new URL(str);
			try {
				String ext = getExtension(url);
				ReaderAccessibility reader = null;
				if (ext.equals("rtf")) {
					reader = new RtfDataReader(url);
				} else if (ext.equals("odt")) {
					reader = new OdtDataReader(url);
				} else if (ext.equals("htm")) {
					reader = new HtmDataReader(url);
				}
				if (reader != null) {
					tokenize(new BufferedReader(new StringReader(reader.getData())));
					cadre.setStructure(toxml(reader.getStructure()));
				} else {
					if (ext.equals("xml")) {
						XMLDataReader dataReader = new XMLDataReader(url);
						tokenize(dataReader);
						cadre.setStructure(toxml(dataReader.getStructure()));
					} else {
						JOptionPane.showMessageDialog(null, "Unexpected extension file = "+ext, "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			} catch (Exception err) {
				JOptionPane.showMessageDialog(null, err.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		} catch(Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	/**
	 * Charge une nouvelle trame et un nouveau cadre  partir d'un fichier.
	 * @param fic     Fichier au format RichText, OpenDocument, HyperText Markup Language ou eXtensible Markup Language
	 * @param ext Extension du fichier, son type (rtf, odt, htm ou xml)
	 **/
	public void openFile(File fic, String ext) {
		try {
			ReaderAccessibility reader = null;
			if (ext.equals("rtf")) {
				reader = new RtfDataReader(fic);
			} else if (ext.equals("odt")) {
				reader = new OdtDataReader(fic);
			} else if (ext.equals("htm")) {
				reader = new HtmDataReader(fic);
			}
			if (reader != null) {
				tokenize(new BufferedReader(new StringReader(reader.getData())));
				cadre.setStructure(toxml(reader.getStructure()));
			} else {
				if (ext.equals("xml")) {
					XMLDataReader dataReader = new XMLDataReader(fic);
					tokenize(dataReader);
					cadre.setStructure(toxml(dataReader.getStructure()));
				} else {
					JOptionPane.showMessageDialog(null, "Unexpected extension file = "+ext, "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	/**
	 * Charge une nouvelle trame et un nouveau cadre  partir d'un fichier.
	 * @param fic     Fichier texte au texte seul
	 * @param charset Encodage des caracteres
	 **/
	public void openFile(File fic, Charset charset) {
		try {
			ReaderAccessibility reader = new TextDataReader(fic, charset);
			tokenize(new BufferedReader(new StringReader(reader.getData())));
			cadre.setStructure(toxml(reader.getStructure()));
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Tokenize .
	 **/
	private void tokenize(BufferedReader dataReader) {
		if (dataReader!=null) {
			trame.reset();
			Tokenizer tokenizer = trame.getNewTokenizer();
			if (tokenizer != null) {
				try {
					long debut = System.currentTimeMillis();

					tokenizer.tokenize(dataReader);
					dataReader.close();

					long tempsTotal = System.currentTimeMillis()-debut;
					System.out.println("Tokens: " + trame.size()+" Total time: "+tempsTotal+ " ms");
					Partition part = computePartition(PreferencesModel.PARTITION_SEPARATORS, "default");
					cadreController.addPartition(part);
					trameController.notifyTrameChanged();
				} catch (Exception e) {
					JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	private void tokenize(XMLDataReader dataReader) {
		if (dataReader!=null) {
			trame.reset();
			Tokenizer tokenizer = trame.getNewTokenizer();
			if (tokenizer != null) {
				try {
					long debut = System.currentTimeMillis();
					dataReader.tokenize(tokenizer);
					long tempsTotal = System.currentTimeMillis()-debut;
					System.out.println("Tokens: " + trame.size()+" Total time: "+tempsTotal+ " ms");

					trameController.notifyTrameChanged();
				} catch (Exception e) {
					JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private String getExtension(URL url) throws Exception {
		HttpURLConnection connection = (HttpURLConnection)url.openConnection();
		if (connection.getResponseCode() == 200) {
			String contentType = connection.getContentType();
			if (contentType.matches("text/html.*")) {
				return "htm";
			} else if (contentType.matches("text/rtf.*")) {
				return "rtf";
			} else if (contentType.matches("text/plain.*")) {
				return "txt";
			} else if (contentType.matches("application/vnd.oasis.opendocument.text.*")) {
				return "odt";
			} else if (contentType.matches("text/xml.*")) {
				return "xml";
			} else {
				throw new Exception("Unexpected contentType = "+contentType);
			}
		} else {
			throw new Exception("Return code = "+connection.getResponseCode());
		}
	}
	private org.w3c.dom.Element toxml(javax.swing.text.Element node) {
		ArrayList positions = trame.getPositions();
		org.w3c.dom.Element root = (org.w3c.dom.Element) cadre.createElement("structure");
		expandElement(node, root, positions);
		return root;
	}
    private void expandElement(javax.swing.text.Element elt, org.w3c.dom.Element parent, ArrayList positions) {
		javax.swing.text.AttributeSet atts = elt.getAttributes();
		String tagname = (String)atts.getAttribute("tagname");
		org.w3c.dom.Element n = (org.w3c.dom.Element) cadre.createElement((tagname == null)? elt.getName(): tagname);

		String max = String.valueOf(positions.get(positions.size()-1));
		String start = max;
		String end = max;
		if (elt.getStartOffset() < positions.size()) {
			start = String.valueOf(positions.get(elt.getStartOffset()));
		}
		if (elt.getEndOffset()-1 < positions.size()) {
			end =  String.valueOf(positions.get(elt.getEndOffset()-1));
		}
		n.setAttribute("start", String.valueOf(start));
		n.setAttribute("end", String.valueOf(end));
		parent.appendChild(n);

        for (int i = 0; i < elt.getElementCount(); i++) {
            javax.swing.text.Element childElement = elt.getElement(i);
            if (!childElement.isLeaf()) {
				expandElement(childElement, n, positions);
            }
        }
    }

	/**
	 * Enregistre la trame textuelle sous forme xml dans un fichier (chaque tnum est ecrit avec son dictionaire associe)
	 **/
	public void import_data(File fic) {
		try {
			trame.read(fic);
		} catch (Exception err) {
			System.err.println(err.getMessage());
		}
	}
	/**
	 * Enregistre la trame textuelle sous forme xml dans un fichier (chaque tnum est ecrit avec son dictionaire associe)
	 **/
	public void export_data(File fic) {
		try {
			trame.write(fic);
		} catch (Exception err) {
			System.err.println(err.getMessage());
		}
	}

	public AbstractSelection getNewSelection(AbstractSelection selectionSelected) {
		String title = JOptionPane.showInputDialog(null, Textometre.LABELS.getLocaliseString("message.selection.name")+":", Textometre.LABELS.getLocaliseString("message.selection.name"), JOptionPane.QUESTION_MESSAGE);
		if ((title != null) && (!title.equals(""))) {
			if (cadreController.containsSelection(title)) {
				int response = JOptionPane.showConfirmDialog(null, Textometre.LABELS.getLocaliseString("message.selection.modify")+": '"+title+"'", JOptionPane.MESSAGE_PROPERTY, JOptionPane.YES_NO_OPTION);
				if (response != JOptionPane.YES_OPTION) {
					return null;
				}
			}
			//il faut creer un nouvel objet AbstractSelection a chaque fois (pour le undo...)
			AbstractSelection sel = new IntervalSelection(title, trameController.getTrameSize());
			return sel.union(selectionSelected);
		}
		return null;
	}
}