/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.cadre;

import socle.*;
import socle.util.*;
import socle.trame.*;
import socle.selection.*;
import socle.partition.*;
import socle.lexicon.article.*;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import org.xml.sax.InputSource;
import javax.swing.event.EventListenerList;

/*-----------------------------------------------------------------------*/

/** Modele de donnees definissant un Cadre de navigation dans une trame textuelle.
* Le cadre possede pour le moment: des partitions et des selections ainsi qu'une structure arborescente si le document en avait
* une au depart (les balises xml)
*/
public class CadreModel {

		private Document                              dom;
		private DocumentBuilder                       builder;
		private Hashtable<String,AbstractSelection>   selections;
		private Hashtable<String,Partition>           partitions;
		private EventListenerList                     listeners;

	/**
	 * Cree un DOM a partir d'un fichier.
	 **/
	public CadreModel() {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			this.builder = factory.newDocumentBuilder();
		} catch (Exception e) {
			this.builder = null;
			System.err.println("Unable to create cadre "+ e.getMessage());
		}
		this.dom = null;
		this.selections = new Hashtable<String,AbstractSelection>();
		this.partitions = new Hashtable<String,Partition>();
		this.listeners = new EventListenerList();
		newDocument();
	}
	/**
	 * Provoque la mise a jour de l'editeur de cadre.
	 **/
	public void fireCadreDomChanged() {
		CadreChangedEvent event = new CadreChangedEvent(this, dom);
		CadreListener[] listenerList = (CadreListener[])listeners.getListeners(CadreListener.class);
		for(int i=0; i<listenerList.length; i++) {
			CadreListener listener = listenerList[i];
			listener.cadreChanged(event);
		}
	}
	public void fireCadreSelectSelection(ArrayList list, ZoneTrame zt) {
		CadreListener[] listenerList = (CadreListener[])listeners.getListeners(CadreListener.class);
		for(int i=0; i<listenerList.length; i++) {
			CadreListener listener = listenerList[i];
			listener.cadreChanged(new CadreSelectEvent(this, list, zt));
		}
	}
	public void fireCadreShowZone(ZoneTrame zone) {
		CadreListener[] listenerList = (CadreListener[])listeners.getListeners(CadreListener.class);
		for(int i=0; i<listenerList.length; i++) {
			CadreListener listener = listenerList[i];
			listener.cadreChanged(new CadreShowEvent(this, zone));
		}
	}
	public void fireCadrePartitionsChanged(ArrayList partitionNames) {
		CadreListener[] listenerList = (CadreListener[])listeners.getListeners(CadreListener.class);
		for(int i=0; i<listenerList.length; i++) {
			CadreListener listener = listenerList[i];
			listener.cadreChanged(new PartitionsChangedEvent(this, partitionNames));
		}
	}
	public void addCadreListener(CadreListener listener) {
		listeners.add(CadreListener.class, listener);
	}
	public void removeCadreListener(CadreListener l) {
		 listeners.remove(CadreListener.class, l);
	}

	/**
	 * Charge le cadre a partir d'un fichier xml.
	 **/
	public void loadFromFile(File fic) {
		Hashtable<String,AbstractSelection> hash = new Hashtable<String,AbstractSelection>();
		try {
			dom = builder.parse(fic);
			Element cadreElt = (Element)dom.getDocumentElement();
			if (cadreElt != null) {
				NodeList sels = cadreElt.getElementsByTagName("selection");
				for (int i=0; i<sels.getLength(); i++) {
					Element sel = (Element)sels.item(i);
					NodeList titles = sel.getElementsByTagName("title");
					Element titleElt = (Element)titles.item(0);
					String tit = titleElt.getTextContent();
					if (!selectionsContains(tit)) {
						hash.put(tit, new IntervalSelection(sel));
					} else {
						System.err.println("Duplicate name '"+tit+"' for selection");
					}
				}
			}
		} catch (Exception e) {
			dom = null;
			System.err.println("Unable  to create cadre "+ e.getMessage());
		}
		selections.clear();
		selections.putAll(hash);
		fireCadreDomChanged();
	}
	/**
	 * Ecrit le contenu du cadre dans un fichier xml.
	 **/
	public void writeToFile(File fic) {
		try {
			Source source = new DOMSource(dom);
			Result result = new StreamResult(fic);
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.transform(source, result);
        } catch (Exception e) {
			System.err.println(e.getMessage());
        }
	}
	/**
	 * Cree un nouveau cadre vide.
	 * Le nouveau cadre a pour structure xml /cadre/structure
	 **/
	public void newDocument() {
		try {
			dom = builder.newDocument();
			Element cadreElt = dom.createElement("cadre");
			dom.appendChild(cadreElt);
		} catch(Exception e) {
			dom = null;
			System.err.println("Unable to create cadre "+ e.getMessage());
		}
		fireCadreDomChanged();
	}
	/**
	 * Cree un nouvel element dans le cadre
	 **/
	public Element createElement(String localName) {
		return dom.createElement(localName);
	}

	/**************
	 * STRUCTURE  *
	 **************/

	/**
	 * Remplace la structure existante par la nouvelle
	 **/
	public void setStructure(Element newStructure) {
		Element cadreElt = (Element)dom.getDocumentElement();
		if (cadreElt != null) {
			NodeList nodelist = cadreElt.getElementsByTagName("structure");
			Element oldStructure = null;
			if (nodelist.getLength() > 0) {
				oldStructure = (Element)nodelist.item(0);
			} else {
				oldStructure = dom.createElement("structure");
				cadreElt.appendChild(oldStructure);
			}
			if (oldStructure != null) {
				cadreElt.replaceChild(newStructure, oldStructure);
			}
		}
		fireCadreDomChanged();
	}
	/**
	 * Retourne la structure actuelle
	 **/
	public Element getStructure() {
		Element cadreElt = (Element)dom.getDocumentElement();
		if (cadreElt != null) {
			NodeList nodelist = cadreElt.getElementsByTagName("structure");
			return (Element)nodelist.item(0);
		}
		return null;
	}

	/**************
	 * PARTITIONS *
	 **************/

	/**
	 * Retourne la partition.
	 * @param title     titre de la partition recherchee
	 **/
	public Partition getPartition(String title) {
		Element part = getPartitionElt(title);
		if (part != null) {
			return new Partition(part);
		} else {
			return null;
		}
	}
	/**
	 * Retourne la liste de tous les noms des partitions.
	 **/
	public ArrayList<String> getPartitionNames() {
		return new ArrayList<String>(partitions.keySet());
	}
	/**
	 * Retourne la partition.
	 * @param title     titre de la partition recherchee
	 **/
	public Element getPartitionElt(String title) {
		Element cadreElt = (Element)dom.getDocumentElement();
		if (cadreElt != null) {
			NodeList parts = cadreElt.getElementsByTagName("partition");
			for (int i=0; i<parts.getLength(); i++) {
				Element part = (Element)parts.item(i);
				NodeList titles = part.getElementsByTagName("title");
				Element titleElt = (Element)titles.item(0);
				String tit = titleElt.getTextContent();
				if (tit.equals(title)) {
					return part;
				}
			}
		}
		return null;
	}
	/**
	 * Teste si une partition de meme titre a deja ete definie.
	 **/
	public boolean partitionsContains(String title) {
		return partitions.containsKey(title);
	}
	/**
	 * Ajoute une partition au cadre.
	 * @param partition     partition a ajouter
	 **/
	public void setPartition(Partition partition) {
		String xmlString = partition.toXML();
		try {
			Document doc = builder.parse(new InputSource(new StringReader(xmlString)));
			Element elt = (Element)doc.getDocumentElement();
			Node node = dom.importNode(elt, true);
			if (partitionsContains(partition.getTitle())) {
				Element oldElt = getPartitionElt(partition.getTitle());
				if (oldElt != null) {
					Node parent = oldElt.getParentNode();
					parent.replaceChild(node, oldElt);
				}
			} else {
				NodeList nodelist = dom.getDocumentElement().getElementsByTagName("partitions");
				if (nodelist.getLength() > 0) {
					nodelist.item(0).appendChild(node);
				} else {
					Element partitionsElt = dom.createElement("partitions");
					partitionsElt.appendChild(node);
					dom.getDocumentElement().appendChild(partitionsElt);
				}
			}
		} catch (Exception e) {
			System.err.println("Unable to add the code "+xmlString+": "+e.getMessage());
		}
		partitions.put(partition.getTitle(), partition);
		fireCadreDomChanged();
		fireCadrePartitionsChanged(getPartitionNames());
	}
	/**
	 * Supprime une partition au cadre.
	 * @param partition     partition a supprimer
	 **/
	public void delPartition(Partition partition) {
		Element oldElt = getPartitionElt(partition.getTitle());
		if (oldElt != null) {
			Node parent = oldElt.getParentNode();
			parent.removeChild(oldElt);
		}
		partitions.remove(partition.getTitle());
		fireCadreDomChanged();
		fireCadrePartitionsChanged(getPartitionNames());
	}

	/**************
	 * SELECTIONS *
	 **************/

	/**
	 * Retourne la selection.
	 * @param title     titre de la selection recherchee
	 **/
	public AbstractSelection getSelection(String title) {
		Element sel = getSelectionElt(title);
		if (sel != null) {
			return new IntervalSelection(sel);
		} else {
			return null;
		}
	}
	/**
	 * Retourne la selection.
	 * @param title     titre de la selection recherchee
	 **/
	private Element getSelectionElt(String title) {
		Element cadreElt = (Element)dom.getDocumentElement();
		if (cadreElt != null) {
			NodeList sels = cadreElt.getElementsByTagName("selection");
			for (int i=0; i<sels.getLength(); i++) {
				Element sel = (Element)sels.item(i);
				NodeList titles = sel.getElementsByTagName("title");
				Element titleElt = (Element)titles.item(0);
				String tit = titleElt.getTextContent();
				if (tit.equals(title)) {
					return sel;
				}
			}
		}
		return null;
	}
	/**
	 * Teste si une selection de meme titre a deja ete definie.
	 **/
	public boolean selectionsContains(String title) {
		return selections.containsKey(title);
	}
	/**
	 * Ajoute une selection au cadre.
	 * @param selection     selection a ajouter
	 **/
	public void setSelection(AbstractSelection selection) {
		String xmlString = selection.toXML();
		try {
			Document doc = builder.parse(new InputSource(new StringReader(xmlString)));
			Element elt = (Element)doc.getDocumentElement();
			Node node = dom.importNode(elt, true);
			if (selectionsContains(selection.getTitle())) {
				Element oldSelElt = getSelectionElt(selection.getTitle());
				if (oldSelElt != null) {
					Node parent = oldSelElt.getParentNode();
					parent.replaceChild(node, oldSelElt);
				}
			} else {
				NodeList nodelist = dom.getDocumentElement().getElementsByTagName("selections");
				if (nodelist.getLength() > 0) {
					nodelist.item(0).appendChild(node);
				} else {
					Element selectionsElt = dom.createElement("selections");
					selectionsElt.appendChild(node);
					dom.getDocumentElement().appendChild(selectionsElt);
				}
			}
		} catch (Exception e) {
			System.err.println("Unable to add the code "+xmlString+": "+e.getMessage());
		}
		selections.put(selection.getTitle(), selection);
		fireCadreDomChanged();
	}
	/**
	 * Supprime une selection au cadre.
	 * @param selection     selection a supprimer
	 **/
	public void delSelection(AbstractSelection selection) {
		Element oldSelElt = getSelectionElt(selection.getTitle());
		if (oldSelElt != null) {
			Node parent = oldSelElt.getParentNode();
			parent.removeChild(oldSelElt);
		}
		selections.remove(selection.getTitle());
		fireCadreDomChanged();
	}
}