/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.cadre;

import socle.*;
import socle.util.*;
import socle.trame.*;
import socle.partition.*;
import socle.selection.*;
import socle.lexicon.article.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.xml.parsers.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.undo.*;
import org.w3c.dom.*;

/*-----------------------------------------------------------------------*/

/** Editeur pour un cadre de navigation dans une trame textuelle.
*/
public class CadreView extends JScrollPane implements CadreListener {

		private CadreController         controller;
		private JTree                   tree;

	/**
	 * Vue d'un cadre de navigation dans une trame textuelle.
	 * Le cadre est presente sous la forme d'un arbre.
	 **/
	public CadreView(CadreController c) {
		super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		Dimension dim = getPreferredSize();
		dim.setSize(150, dim.getHeight());
		setPreferredSize(dim);

		this.controller = c;
		this.tree = new JTree();
		tree.getSelectionModel().setSelectionMode( TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION );
		tree.setShowsRootHandles(true);
		//setEditable(true);
		tree.setModel(null);
		try {
			tree.setLargeModel(true);//?
			tree.addMouseListener( new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					int selRow = tree.getRowForLocation(e.getX(), e.getY());
					final DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
					if (node == null) return;

					//selection la zone dans le texte
					if (node.getUserObject() instanceof ZoneTrame) {
						ZoneTrame sel = (ZoneTrame)node.getUserObject();
						controller.showTrame(sel);
					}
					//popup pour supprimer une selection ou une partition
					if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
						JPopupMenu popup = new JPopupMenu();
						Action actionDel = null;
						if (node.getUserObject() instanceof AbstractSelection) {
							AbstractSelection sel = (AbstractSelection)node.getUserObject();
							actionDel = new RemoveSelectionAction(sel);
						} else if (node.getUserObject() instanceof Partition) {
							Partition part = (Partition) node.getUserObject();
							actionDel = new RemovePartitionAction(part);
						}
						if (actionDel != null) {
							JMenuItem menuDel = new JMenuItem(actionDel);
							popup.add(menuDel);
							popup.show(tree, e.getX(), e.getY());
						}
					}
				}
			});
			setViewportView(tree);
			DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
				public synchronized void drop (DropTargetDropEvent dropTargetDropEvent) {
					try {
						Transferable tr = dropTargetDropEvent.getTransferable();
						DataFlavor serialArrayListFlavor = new DataFlavor(ArrayList.class, "ArrayList");
						if (tr.isDataFlavorSupported (serialArrayListFlavor)) {
							dropTargetDropEvent.acceptDrop (DnDConstants.ACTION_COPY);
							ArrayList list = (ArrayList)tr.getTransferData(serialArrayListFlavor);

							Point e = dropTargetDropEvent.getLocation();
							int selRow = tree.getRowForLocation((int)e.getX(), (int)e.getY());
							TreePath selPath = tree.getPathForLocation((int)e.getX(), (int)e.getY());
							DefaultMutableTreeNode node = (DefaultMutableTreeNode)selPath.getLastPathComponent();
							if ((node != null) && (node.getUserObject() instanceof ZoneTrame)) {
								ZoneTrame zt = (ZoneTrame)node.getUserObject();
								controller.selectTrame(list, zt);
							}
							dropTargetDropEvent.getDropTargetContext().dropComplete(true);
					  } else {
						System.err.println("Rejected");
						dropTargetDropEvent.rejectDrop();
					  }
					} catch (UnsupportedFlavorException e) {
						e.printStackTrace();
						dropTargetDropEvent.rejectDrop();
					} catch (Exception err) {
						err.printStackTrace();
						dropTargetDropEvent.rejectDrop();
					}
				}
			};
			DropTarget dropTarget = new DropTarget (tree, dropTargetAdapter);
			tree.setDragEnabled(true);
			SelectionTransferHandler arrayListHandler = new SelectionTransferHandler();
			tree.setTransferHandler(arrayListHandler);
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

   /**
    * Prend un noeud du DOM et parcours recursivement tous ses enfants en les ajoutant a un
    * DefaultMutableTreeNode..
    */
	private DefaultMutableTreeNode createTreeNode( Node current ) {
		DefaultMutableTreeNode  treeNode = null;
		NamedNodeMap attributes = current.getAttributes();

		//Si c'est un element avec des attributs start et end on cree un noeud de type ZoneTrame
		//si c'est un noeud zone avec des attributs from et until on cree un noeud de type ZoneTrame
		//sinon on cree un noeud portant juste le nom du tag ou sa valeur
		if (current.getNodeType() == Node.ELEMENT_NODE) {
			if(( attributes != null ) && (attributes.getNamedItem("start")!=null) && (attributes.getNamedItem("end")!=null)){
				String start = attributes.getNamedItem("start").getNodeValue().trim();
				String end = attributes.getNamedItem("end").getNodeValue().trim();
				ZoneTrame zone = new ZoneTrame(start, end);
				zone.setTitle(current.getNodeName());
				treeNode = new DefaultMutableTreeNode(zone);
			} else if (current.getNodeName().equals("selection")) {
				AbstractSelection selection = new IntervalSelection((Element)current);
				treeNode = new DefaultMutableTreeNode(selection);
			} else if (current.getNodeName().equals("partition")) {
				Partition partition = new Partition((Element)current);
				treeNode = new DefaultMutableTreeNode(partition);
			} else if(current.getNodeName().equals("zone") && (attributes.getNamedItem("from")!=null) && (attributes.getNamedItem("until")!=null)){
				String start = attributes.getNamedItem("from").getNodeValue().trim();
				String end = attributes.getNamedItem("until").getNodeValue().trim();
				ZoneTrame zone = new ZoneTrame(start, end);
				zone.setTitle(current.getNodeName());
				treeNode = new DefaultMutableTreeNode(zone);
			} else {
				treeNode = new DefaultMutableTreeNode(current.getNodeName());
			}
		} else {
			treeNode = new DefaultMutableTreeNode( current.getNodeType() == Node.TEXT_NODE ? current.getNodeValue() : current.getNodeName());
		}

		//on arrete l'affichage de l'arborescence pour les selections, les partitions
		if( current.hasChildNodes()
			 && (!current.getNodeName().equals("selection"))
			) {
			NodeList children = current.getChildNodes();
			if( children != null ) {
				int numChildren = children.getLength();
				for (int i=0; i < numChildren; i++) {
					Node node = children.item(i);
					if( node != null ) {
						if( node.getNodeType() == Node.ELEMENT_NODE ) {
							treeNode.add( createTreeNode(node) );
						}
						String data = node.getNodeValue();
						if( data != null ) {
							data = data.trim();
							if ( !data.equals("\n") && !data.equals("\r\n") && data.length() > 0 ) {
								treeNode.add(createTreeNode(node));
							}
						}
					}
				}
			}
		}
		return treeNode;
	}

	/********************************************************************
	**   Ecoute les evenements de modifications en provenance du cadre
	*********************************************************************/
	public void cadreChanged(CadreEvent event) {
		if (event instanceof CadreChangedEvent) {
			CadreChangedEvent e = (CadreChangedEvent)event;
			DefaultMutableTreeNode root = createTreeNode(e.getNewCadre());
			tree.setModel( new DefaultTreeModel(root) );
		}
	}

	/********************************************************************
	**   Action de suppression de selection dans le cadre
	*********************************************************************/
	public class RemoveSelectionAction extends AbstractAction {
			AbstractSelection             selection;

		public RemoveSelectionAction(AbstractSelection sel) {
			super(Textometre.LABELS.getLocaliseString("action.remove.selection")+" '"+sel.getTitle()+"'");
			this.selection = sel;
		}
		public void addContent(AbstractSelection sel) {
			controller.addSelection(sel);
		}
		public void removeContent(AbstractSelection sel) {
			controller.delSelection(sel);
		}
		public void actionPerformed(ActionEvent e) {
			UndoableEdit edit = new RemoveSelectionEdit(this, selection);
 			Textometre.UNDO_SUPPORT.postEdit(edit);
		}
	}
	/********************************************************************
	**   Action de suppression de partition dans le cadre
	*********************************************************************/
	public class RemovePartitionAction extends AbstractAction {
			Partition             partition;

		public RemovePartitionAction(Partition part) {
			super(Textometre.LABELS.getLocaliseString("action.remove.partition")+" '"+part.getTitle()+"'");
			this.partition = part;
		}
		public void addContent(Partition part) {
			controller.addPartition(part);
		}
		public void removeContent(Partition part) {
			controller.delPartition(part);
		}
		public void actionPerformed(ActionEvent e) {
			UndoableEdit edit = new RemovePartitionEdit(this, partition);
			Textometre.UNDO_SUPPORT.postEdit(edit);
		}
	}
	/********************************************************************
	** Action reversible de supression d'une selection.
	*********************************************************************/
	public class RemoveSelectionEdit extends AbstractUndoableEdit {

			private RemoveSelectionAction action;
			private AbstractSelection     sel_new;

		public RemoveSelectionEdit(Object obj, AbstractSelection sel_new) {
			this.action = (RemoveSelectionAction)obj;
			this.sel_new = sel_new;
			action.removeContent(sel_new);
		}
		public void undo() throws CannotUndoException {
			super.undo();
			action.addContent(sel_new);
		}
		public void redo() throws CannotRedoException {
			super.redo();
			action.removeContent(sel_new);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			return Textometre.LABELS.getLocaliseString("action.remove.selection");
		}

	}
	/********************************************************************
	** Action reversible de supression d'une partition.
	*********************************************************************/
	public class RemovePartitionEdit extends AbstractUndoableEdit {

			private RemovePartitionAction action;
			private Partition             part_new;

		public RemovePartitionEdit(Object obj, Partition part_new) {
			this.action = (RemovePartitionAction)obj;
			this.part_new = part_new;
			action.removeContent(part_new);
		}
		public void undo() throws CannotUndoException {
			super.undo();
			action.addContent(part_new);
		}
		public void redo() throws CannotRedoException {
			super.redo();
			action.removeContent(part_new);
		}
		public boolean canUndo() {
			return true;
		}
		public boolean canRedo() {
			return true;
		}
		public String getPresentationName() {
			return Textometre.LABELS.getLocaliseString("action.remove.partition");
		}
	}
}