/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon;

import socle.trame.*;
import socle.lexicon.article.*;
import socle.preferences.*;

import java.awt.*;
import java.awt.event.*;
/*-----------------------------------------------------------------------*/

/**
 * Controleur pour un lexique.
 **/
public class LexController {

	private LexView           lexView;
	private LexMenuView       lexMenuView;
	private String            menuItemName;
	private LexModel          model;

	/**
	 * Creation du controleur.
	 * @param model                Modele de donnees d'un lexique.
	 **/
	public LexController (LexModel model) {
		this.model = model;
		this.lexView = new LexView(this);
		this.menuItemName = null;
		model.addLexListener(lexView);
		this.lexMenuView = new LexMenuView();
		lexMenuView.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeLexNameAction();
			}
		});
	}

	/**
	 * Le menu de choix du lexique a ete active .
	 **/
	private void changeLexNameAction() {
		String name = (String)lexMenuView.getSelectedItem();
		if ((name!=null) && (!name.equals(menuItemName))) {
			menuItemName = name;
			model.fireLexNameChanged(name);
		}
	}

	/**
	 * Ajoute la vue comme PreferenceListener.
	 **/
	public void addPreferencesListener(PreferencesModel preferencesModel) {
		preferencesModel.addPreferencesListener(lexView);
	}
	/**
	 * Ajoute la vue comme TrameListener.
	 **/
	public void addTrameListener(TrameModel trame) {
		trame.addTrameListener(lexMenuView);
	}

	/**
	 * Ajoute les vues sur le lexique dans le conteneur specifie.
	 **/
	public void displayView(Container container) {
		container.add(lexMenuView, java.awt.BorderLayout.NORTH);
		container.add(lexView, java.awt.BorderLayout.CENTER);
	}

	/**
	 * Change dans le lexique la forme graphique d'un article par une autre.
	 * Si cette forme est une forme deja existante, les deux articles sont runis
	 **/
	public void doChangeGraph(String oldValue, String newValue) {
		try {
			Article art_old = model.getArticle(oldValue);
			int numap_old = art_old.getNumap();
			try {
				Article art_new = model.getArticle(newValue);
				//on supprime l'ancien et on augmente la frequence de la cible
				model.fireMergeArticlesEvent(numap_old, art_new.getNumap(), oldValue, newValue);
			} catch (Exception err) {
				//on change juste la forme
				model.fireChangeFormArticleEvent(numap_old, oldValue, newValue);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Retourne l'article correspondant a un numero d'ordre.
	 * @param numap     Numero d'apparition
	 *
	 * @return l'article de numero d'ordre num
	 * @throws Exception si le numero d'ordre n'existe pas dans le lexique
	 **/
	public Article getArticle(int numap) throws Exception {
		return model.getArticle(numap);
	}

	/**
	 * Retourne l'article correspondant a la forme graphique.
	 * @param graph     Forme graphique
	 *
	 * @return l'article
	 * @throws Exception si le numero d'ordre n'existe pas dans le lexique
	 **/
	public Article getArticle(String graph) throws Exception {
		return model.getArticle(graph);
	}

	/**
	 * Fixe le modele de donnee a partir du lexique du tnum specifie puis envoi un evenement de modification du lexique.
	 **/
	public void setModel(Tnum tnum) {
		model.setModel(tnum);
	}

	/**
	 * Retourne le nom du lexique.
	 **/
	public String getName() {
		return model.getName();
	}
}
