/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon;

import socle.*;
import socle.trame.*;
import socle.lexicon.article.*;
import java.util.*;
import java.io.Serializable;
import javax.swing.event.EventListenerList;
/*-----------------------------------------------------------------------*/

/**
 * La classe <code>LexModel</code> definit le modele de donnees pour un lexique (ensemble d'items de type <code>Article</code>).
 **/
public class LexModel implements Serializable  {
		private HashMap<Integer,Article>  dicoNum;
		private HashMap<String,Article>   dicoGraph;
		private EventListenerList         listeners;
		private String                    name;

	/**
	 * Cree un nouveau lexique vide.
	 **/
	public LexModel() {
		this.dicoGraph = new HashMap<String,Article>(15000);
		this.dicoNum = new HashMap<Integer,Article>(15000);
		this.listeners = new EventListenerList();
		this.name = "";
	}

	/**
	 * Vide le lexique.
	 **/
	protected void reset() {
		dicoGraph.clear();
		dicoNum.clear();
	}

	/**
	 * Retourne le nom du lexique.
	 **/
	protected String getName() {
		return name;
	}

	/**
	 * Recharge le modele de donnee a partir du lexique du tnum specifie.
	 **/
	protected void setModel(Tnum tnum) {
		reset();
		Set forms = tnum.getAllForms();
		Iterator iterator = forms.iterator();
		while (iterator.hasNext()) {
			Object o = iterator.next();
			Integer numap = tnum.getNumap(o);
			int freq = tnum.getFreq(numap);
			String graph = tnum.getGraph(numap);
			addArticle(numap.intValue(), graph, freq);
		}
		fireLexChanged();
	}

	/**
	 * Ajoute un article au lexique.
	 * @param numap        Numero d'apparition
	 * @param graph        Forme graphique
	 * @param freq         Frequence
	 **/
	private void addArticle(int numap, String graph, int freq) {
		Article art = new Article(graph, numap);
		art.setFreq(freq);
		dicoGraph.put(graph, art);
		dicoNum.put(new Integer(numap), art);
	}

	/**
	 * Retourne l'article correspondant a un numero d'ordre.
	 * @param numap     Numero d'apparition
	 *
	 * @return l'article de numero d'ordre num
	 * @throws Exception si le numero d'ordre n'existe pas dans le lexique
	 **/
	protected Article getArticle(int numap) throws Exception {
		Article art = dicoNum.get(new Integer(numap));
		if (art != null) {
			return art;
		} else {
			throw new Exception("the key "+numap+" is not mapped to any value in the lexicon");
		}
	}
	/**
	 * Retourne l'article correspondant a la forme graphique.
	 * @param graph     Forme graphique
	 *
	 * @return l'article
	 * @throws Exception si le numero d'ordre n'existe pas dans le lexique
	 **/
	protected Article getArticle(String graph) throws Exception {
		Article art = dicoGraph.get(graph);
		if (art != null) {
			return art;
		} else {
			throw new Exception("'"+graph+"' is not a value of a lexicon entry");
		}
	}

	/**
	 * Calcule et fixe le rang lexicometrique des articles.
	 **/
	private void setLexicomOrder() {
		ArrayList<Article> keys = new ArrayList<Article>();
		keys.addAll(dicoNum.values());
		Collections.sort(keys, TrameModel.LEXICOM_ORDER);
		Iterator iterator = keys.iterator();
		int i=0;
		while (iterator.hasNext()) {
			Article art = (Article)iterator.next();
			art.setLexicom(i);
			i++;
		}
	}
	/**
	 * Calcule et fixe le rang lexicographique des articles.
	 **/
	private void setLexicogOrder() {
		ArrayList<Article> keys = new ArrayList<Article>();
		keys.addAll(dicoNum.values());
		Collections.sort(keys, TrameModel.LEXICOG_ORDER);
		Iterator iterator = keys.iterator();
		int i=0;
		while (iterator.hasNext()) {
			Article art = (Article)iterator.next();
			art.setLexicog(i);
			i++;
		}
	}

	/**
	 * Envoi un evenement decrivant le changement de forme d'un article.
	 **/
	protected void fireChangeFormArticleEvent(int numap, String old_graph, String new_graph) {
		ReplaceArticleEvent event = new ReplaceArticleEvent(this, numap, numap, old_graph, new_graph);
		fireEvent(event);
	}

	/**
	 * Envoi un evenement decrivant la fusion de deux articles.
	 **/
	protected void fireMergeArticlesEvent(int old_numap, int new_numap, String old_graph, String new_graph) {
		ReplaceArticleEvent event = new ReplaceArticleEvent(this, old_numap, new_numap, old_graph, new_graph);
		fireEvent(event);
	}

	/**
	 * Envoi un evenement decrivant le changement de nom du lexique.
	 **/
	protected void fireLexNameChanged(String new_name) {
		if (!name.equals(new_name)) {
			name = new_name;
			LexNameChangedEvent event = new LexNameChangedEvent(this, dicoNum.values(), new_name);
			fireEvent(event);
		}
	}

	/**
	 * Envoi un evenement de mise a jour globale du lexique.
	 **/
	protected void fireLexChanged() {
		LexChangedEvent event = new LexChangedEvent(this, dicoNum.values());
		fireEvent(event);
	}

	/**
	 * Envoi un evenement a tous les LexListener.
	 **/
	private void fireEvent(EventObject event) {
		LexListener[] listenerList = (LexListener[])listeners.getListeners(LexListener.class);
		for(int i=0; i<listenerList.length; i++) {
			LexListener listener = listenerList[i];
			listener.lexChanged(event);
		}
	}

	/**
	 * Ajoute un LexListener au lexique.
	 **/
	public void addLexListener(LexListener listener) {
		listeners.add(LexListener.class, listener);
	}

	/**
	 * Supprime le LexListener du lexique.
	 **/
	public void removeLexListener(LexListener l) {
		 listeners.remove(LexListener.class, l);
	}
}
