/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon;

import socle.*;
import socle.preferences.*;
import socle.trame.*;
import socle.preferences.*;
import socle.lexicon.article.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;
/*-----------------------------------------------------------------------*/

/** La classe <code>LexView</code> definit une vue pour un lexique.
*
* <p>La vue <code>LexView</code> est une table qui comporte deux colones: (
* la colone forme graphique
* et la colone frequence des unites).
* le nombre de ligne est egal au nombre d'unites du lexique</p>
*/
public class LexView extends JScrollPane implements LexListener, PreferencesListener {

		private JTable                lexiqueTable;
		private LexController         controller;
		private String                fontName;

	/**
	 * Cree un panel dans lequel un lexique pourra s'afficher dans une table a deux colones ('forme graphique' et 'frequence').
	 **/
	public LexView(LexController controller) {
		super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		Dimension dim = getPreferredSize();
		dim.setSize(150, dim.getHeight());
		setPreferredSize(dim);

		this.controller = controller;
		this.lexiqueTable = new JTable();
		this.fontName = PreferencesModel.FONT_NAME;
		setViewportView(lexiqueTable);
	}

	/**
	 * Fixe la police de caracteres utilisee pour l'affichage.
	 * @param name     nom de la police de caracteres
	 **/
	private void setFontName(String name) {
		fontName = name;
		Font font = lexiqueTable.getFont();
		lexiqueTable.setFont(new Font(fontName, font.getStyle(), font.getSize()));
	}

	/**
	 * Capture des evenements de changement dans le lexique.
	 **/
	public void lexChanged(EventObject event) {
		Collection coll = null;
		if (event instanceof LexNameChangedEvent) {
			LexNameChangedEvent e = (LexNameChangedEvent)event;
			coll = e.getValues();
		} else if (event instanceof LexChangedEvent) {
			LexChangedEvent e = (LexChangedEvent)event;
			coll = e.getValues();
		}
		if (coll != null) {
			final Object[]   columnNames = new Object[] {"graph", "freq"};
			final Object data[][]        = new Object[coll.size()][2];
			Iterator list = coll.iterator();

			int num = 0;
			while ( list.hasNext() ) {
				Article art = (Article)list.next();
				data[num][0] = new Graph(art);
				data[num][1] = new Freq(art);
				num++;
			}
			TableModel model = new AbstractTableModel() {
				public int getColumnCount() {
					return data[0].length;
				}
				public int getRowCount() {
					return data.length;
				}
				public String getColumnName(int col) {
					return (String)columnNames[col];
				}
				public Object getValueAt(int row, int col) {
					return data[row][col];
				}
				public boolean isCellEditable(int row, int col) {
					return (col == 0);//Seule la colonne de la forme graphique est editable;
				}
				public Class getColumnClass(int col) {
					return getValueAt(0, col).getClass();
				}
				public void setValueAt(Object val, int row, int col) {
					if (col == 0) {
						String value = (String)val;
						Article art = ((Graph)data[row][col]).getArticle();
						if (!(art.getGraph().equals(value))) {
							controller.doChangeGraph(art.getGraph(), value);
						}
					}
				}
			};
			if(model.getRowCount()!=0) {
				lexiqueTable = new JTable();
				lexiqueTable.setDragEnabled(true);
				ArticleTransferHandler arrayListHandler = new ArticleTransferHandler();
				lexiqueTable.setTransferHandler(arrayListHandler);
				Font font = lexiqueTable.getFont();
				lexiqueTable.setFont(new Font(fontName, font.getStyle(), font.getSize()));
				myTableSorter sorter = new myTableSorter(model, lexiqueTable.getTableHeader());
				lexiqueTable.setModel(sorter);
				lexiqueTable.setDefaultEditor(Graph.class, new GraphEditor(new JTextField(), JTextField.RIGHT));
				lexiqueTable.getTableHeader().setToolTipText("Click to specify sorting; Control-Click to specify secondary sorting");
				setViewportView(lexiqueTable);
			} else {
				lexiqueTable = new JTable();
				setViewportView(lexiqueTable);
			}
		}
	}
	/**
	 * Capture des evenements de changement dans les preferences.
	 **/
	public void preferencesChanged(PreferencesChangedEvent event) {
		if(event instanceof PreferencesChangedFontEvent) {
			PreferencesChangedFontEvent e = (PreferencesChangedFontEvent)event;
			setFontName(e.getNewFontName());
		}
	}

	/*********************************
	** Champs editeur pour la cellule
	** de la forme graphique d'un article.
	**********************************/
	private class GraphEditor extends DefaultCellEditor implements TableCellEditor {
		public GraphEditor(JTextField textField, int direction) {
			super(textField);
			textField.setDragEnabled(true);

			setClickCountToStart(2);
			final Component comp = textField;
			Font font = textField.getFont();
			textField.setFont(new Font(fontName, Font.PLAIN, font.getSize()));
			textField.setHorizontalAlignment(direction);
		}
	}
	/*********************************
	** Association de l'ordre de tri choisit, aux objects
	** de la colonne 'frequence' du lexique.
	**********************************/
    private class Freq extends Object implements Comparable {
			private Article art;

		public int compareTo(Object o) {
			Comparator<Article> comp = TrameModel.LEXICOM_ORDER;
			return comp.compare(art, ((Freq)o).getArticle());
		}
        public Freq(Article art) {
            this.art = art;
        }
        public Article getArticle() {
			return art;
		}
        public String toString() {
			return String.valueOf(art.getFreq());
		}
    }
}