/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.lexicon.article;

import socle.trame.*;

import java.util.HashMap;
import javax.swing.event.EventListenerList;
/*-----------------------------------------------------------------------*/

/** La classe <code>OccurrenceModel</code> definit une occurrence lexicale.
*
* <p>Cette classe sert a representer des occurences lexicales. Une occurrence se caracterise par:
* sa forme graphique et sa position dans la trame.
* Il peut se voir affecter un numero d'ordre pouvant servir d'identifiant, une frequence
* et des rangs lexicometriques et lexicographiques.</p>
*/
public class OccurrenceModel extends Article {
		private int                     numocc;
		private HashMap<String,String>  properties;
		private EventListenerList       listeners;

	public OccurrenceModel(String graph, int numap, int numocc, HashMap<String,String> properties) {
		this(graph, numap, numocc);
		this.properties = properties;
	}

	/** Initialise une nouvelle occurrence.
	* @param graph     Representation graphique de l'article.
	* @param numap     Numero d'ordre de l'article.
	* @param numocc    Numero de l'occurrence dans la trame textuelle.
	*/
	public OccurrenceModel(String graph, int numap, int numocc) {
		super(graph, numap);
		setFreq(1);
		this.numocc    = numocc;
		this.properties = new HashMap<String,String>();
		properties.put(Tnum.GRAPH, graph);
		this.listeners = new EventListenerList();
	}

	/** Initialise une nouvelle occurrence.
	* @param art       Article du lexique.
	* @param numocc    Numero de l'occurrence dans la trame textuelle.
	*/
	public OccurrenceModel(Article art, int numocc) {
		this(art.getGraph(), art.getNumap(), numocc);
	}

	/** Initialise une nouvelle occurrence.
	* @param occ       Occurrence.
	*/
	public OccurrenceModel(OccurrenceModel occ) {
		this(occ, occ.getNumocc());
		properties = occ.properties;
	}

	/** Fixe les proprietes de l'ccurence dont la valeur de la forme graphique.
	*/
	public void setProperties(HashMap<String,String> map) {
		properties = map;
		super.setGraph(getValue(Tnum.GRAPH));
	}

	/** Retourne le numero de l'occurrence.
	*/
	public int getNumocc() {
		return numocc;
	}

	/** Retourne la liste des proprietes de l'occurrence.
	*/
	public HashMap<String,String> getProperties() {
		return properties;
	}
	/** Retourne la valeur de l'occurrence pour la  propriete specifiee.
	*/
	public String getValue(String key) {
		return properties.get(key);
	}

	/** Retourne une reprsentation sous forme de chaine de caractre.utile pour debuger.
	 **/
	public String toString() {
		return "[occurrence "
		+ numocc
		+ properties
		+" "
		+ super.toString()
		+ "]";
	}

	/** Provoque la mise a jour de l'editeur de la trame.
	 **/
	public void fireOccChanged(){
		OccListener[] listenerList = (OccListener[])listeners.getListeners(OccListener.class);
		for(int i=0; i<listenerList.length; i++) {
			OccListener listener = listenerList[i];
			listener.occChanged(new OccChangedEvent(this, numocc));
		}
	}
	public void addOccListener(OccListener listener){
		listeners.add(OccListener.class, listener);
	}
	public void removeOccListener(OccListener listener){
		 listeners.remove(OccListener.class, listener);
	}
}
