/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2003 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;


/** Table triable.
*/
public class myTableSorter extends AbstractTableModel {

    public static final int UNSORTED =  0;
    public static final int UP       =  1;
    public static final int DOWN     = -1;

    public static final int NONE     = -1;

    protected TableModel       myTableModel  = null;
    private JTableHeader       myTableHeader = null;
    private List<Directive>    sortingColumns = new ArrayList<Directive>();
    private Row[] viewToModel;
    private int[] modelToView;


    /*contructor*/
    public myTableSorter(TableModel tableModel, JTableHeader tableHeader) {
        if (tableHeader != null) {
			this.myTableHeader = tableHeader;
            this.myTableHeader.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					JTableHeader h = (JTableHeader) e.getSource();
					TableColumnModel columnModel = h.getColumnModel();
					int viewColumn = columnModel.getColumnIndexAtX(e.getX());
					int column = columnModel.getColumn(viewColumn).getModelIndex();
					if (column != -1) {
						int status = getSortingStatus(column);
						if (!e.isControlDown()) {
							cancelSorting();
						}
						// Cycle the sorting states through {UNSORTED, UP, DOWN} or
						// {UNSORTED, DOWN, UP} depending on whether shift is pressed.
						status = status + (e.isShiftDown() ? -1 : 1);
						status = (status + 1) % 3 - 1; // signed mod returning {-1, 0, 1}
						setSortingStatus(column, status);
					}
				}
			});
            this.myTableHeader.setDefaultRenderer(new HeaderRenderer(this.myTableHeader.getDefaultRenderer()));
        }
        if (tableModel != null) {
            this.myTableModel = tableModel;
            this.myTableModel.addTableModelListener(new TableModelListener() {
				public void tableChanged(TableModelEvent e) {
					// If we're not sorting by anything, just pass the event along.
					if (sortingColumns.isEmpty()) {
						fireTableChanged(e);
						return;
					}
					// If the table structure has changed, cancel the sorting; the sorting columns may have been either moved or deleted from the model.
					if (e.getFirstRow() == TableModelEvent.HEADER_ROW) {
						cancelSorting();
						fireTableChanged(e);
						return;
					}
					int first = e.getFirstRow();
					int last = e.getLastRow();
					int type = e.getType();
					int column = e.getColumn();
					//if the changes occurs in a sigle row and column
					if ((first == last) && (column != TableModelEvent.ALL_COLUMNS)) {
						//if the changes occurs in an unsorted column + verify if modelToView is already allocated
						if ((getSortingStatus(column) == UNSORTED) && (modelToView != null)) {
							int viewIndex = getModelToView()[first];
							fireTableChanged(new TableModelEvent(myTableSorter.this, viewIndex, viewIndex, column, e.getType()));
							return;
						}
					}
					clearSortingState();
					fireTableDataChanged();
					return;
				}
			});
        }
        clearSortingState();
        fireTableStructureChanged();
    }

    private void clearSortingState() {
        viewToModel = null;
        modelToView = null;
    }

    private Directive getDirective(int column) {
        for (int i = 0; i < sortingColumns.size(); i++) {
            Directive directive = sortingColumns.get(i);
            if (directive.col == column) {
                return directive;
            }
        }
        return new Directive(NONE, UNSORTED);
    }

    public int getSortingStatus(int column) {
        return getDirective(column).direction;
    }
    private void sortingStatusChanged() {
        clearSortingState();
        fireTableDataChanged();
        myTableHeader.repaint();
    }
    public void setSortingStatus(int column, int status) {
        Directive directive = getDirective(column);
		if (directive.exist()) {
            sortingColumns.remove(directive);
		}
        if (status != UNSORTED) {
            sortingColumns.add(new Directive(column, status));
        }
        sortingStatusChanged();
    }

    private void cancelSorting() {
        sortingColumns.clear();
        sortingStatusChanged();
    }

    private Row[] getViewToModel() {
        if (viewToModel == null) {
            int tableModelRowCount = this.myTableModel.getRowCount();
            viewToModel = new Row[tableModelRowCount];
            for (int row = 0; row < tableModelRowCount; row++) {
                viewToModel[row] = new Row(row);
            }

            if (!sortingColumns.isEmpty()) {
                Arrays.sort(viewToModel);
            }
        }
        return viewToModel;
    }

    public int modelIndex(int viewIndex) {
        return getViewToModel()[viewIndex].modelIndex;
    }

    private int[] getModelToView() {
        if (modelToView == null) {
            int n = getViewToModel().length;
            modelToView = new int[n];
            for (int i = 0; i < n; i++) {
                modelToView[modelIndex(i)] = i;
            }
        }
        return modelToView;
    }

    // Les methodes interface
    public int getRowCount() {
        return (this.myTableModel == null) ? 0 : this.myTableModel.getRowCount();
    }
    public int getColumnCount() {
        return (this.myTableModel == null) ? 0 : this.myTableModel.getColumnCount();
    }
    public Object getValueAt(int row, int column) {
        return this.myTableModel.getValueAt(modelIndex(row), column);
    }
    public String getColumnName(int column) {
        return this.myTableModel.getColumnName(column);
    }
    public Class<?> getColumnClass(int column) {
        return this.myTableModel.getColumnClass(column);
    }
    public boolean isCellEditable(int row, int column) {
        return this.myTableModel.isCellEditable(modelIndex(row), column);
    }
    public void setValueAt(Object aValue, int row, int column) {
        this.myTableModel.setValueAt(aValue, modelIndex(row), column);
    }


    /**********************************************************
    ****                    Helper classes
    ***********************************************************/
    private class HeaderRenderer implements TableCellRenderer {
        private TableCellRenderer tableCellRenderer;

        public HeaderRenderer(TableCellRenderer tableCellRenderer) {
            this.tableCellRenderer = tableCellRenderer;
        }

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            Component c = tableCellRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            if (c instanceof JLabel) {
                JLabel l = (JLabel) c;
                l.setHorizontalTextPosition(JLabel.LEFT);
                int modelColumn = table.convertColumnIndexToModel(column);
                Directive dir = getDirective(modelColumn);
				int level = sortingColumns.indexOf(dir);
				if (dir.direction == myTableSorter.UP) {
					l.setText(l.getText() + " \u2191 (" + String.valueOf(level+1) +")");
				} else if (dir.direction == myTableSorter.DOWN) {
					l.setText(l.getText() + " \u2193 (" +String.valueOf(level+1) +")");
				}
            }
            return c;
        }
    }


    private class Row implements Comparable {
        private int modelIndex;

        public Row(int index) {
            this.modelIndex = index;
        }

        public int compareTo(Object o) {
            int row1 = modelIndex;
            int row2 = ((Row) o).modelIndex;

            for (Iterator<Directive> it = sortingColumns.iterator(); it.hasNext();) {
                Directive directive = it.next();
                int column = directive.col;
                Object o1 = myTableModel.getValueAt(row1, column);
                Object o2 = myTableModel.getValueAt(row2, column);

                int comparison = 0;
                // Define null less than everything, except null.
                if (o1 == null && o2 == null) {
                    comparison = 0;
                } else if (o1 == null) {
                    comparison = -1;
                } else if (o2 == null) {
                    comparison = 1;
                } else {
			        Class columnClass = myTableModel.getColumnClass(column);
			        if (Comparable.class.isAssignableFrom(columnClass)) {
						comparison = ((Comparable) o1).compareTo(o2);
					} else {
						comparison = o1.toString().compareTo(o2.toString());
					}
                }
                if (comparison != 0) {
                    return directive.direction == DOWN ? -comparison : comparison;
                }
            }
            return 0;
        }
    }


    private static class Directive {
        private int col;
        private int direction;

        public Directive(int column, int direction) {
            this.col = column;
            this.direction = direction;
        }
        public boolean exist() {
			return (col != NONE);
		}
    }
}