/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2006 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.preferences;

import socle.*;
import socle.trame.*;
import java.util.Locale;
import java.awt.Color;
import javax.swing.event.EventListenerList;
/*-----------------------------------------------------------------------*/

/** Preferences du programme.
*/
public class PreferencesModel  {
		private String                 partitionSeparators;
		private String                 strTokenizer;
		private Class                  classTokenizer = null;
		private Color                  bgColor;
		private String                 fontName;
		private String                 hint;
		private Object[]               typeDefinition;
		private Locale                 locale4collator;
		private Locale                 locale;
		private int                    concContextSize;
		private EventListenerList      listeners;
		private PreferencesResourceBundle         labels;

		public static String           DEFAULT_CLASS_TOKENIZER = Textometre.getProperty("CLASS_TOKENIZER");
		public static String           PARTITION_SEPARATORS    = Textometre.getProperty("PARTITION_SEPARATORS");
		public static Color            BG_COLOR   = Color.red;
		public static String           STR_TOKENIZER   = " \t,.;?!\u00A7\r\n";
		public static String           FONT_NAME   = "";
		public static String           DEFAULT_FACET   = "default";
		public static String           HINT   = DEFAULT_FACET;
		public static Locale           LOCALE   = Locale.getDefault();
		public static int              CONC_CONTEXT_SIZE = 10;
		public static int              CONC_TEXT_SCOPE = 0;
		public static int              CONC_SAME_LEVEL_SCOPE = 1;
		public static String           STYLE_HIGHTLIGHT   = "hightlight";
		public static String           STYLE_UNHIGHTLIGHT = "unhightlight";

	/** Modele de donnees pour les Preferences.
	*/
	public PreferencesModel() {
		this.bgColor             = BG_COLOR;
		this.fontName            = FONT_NAME;
		this.hint                = HINT;
		this.locale              = LOCALE;
		this.locale4collator     = LOCALE;
		this.strTokenizer        = STR_TOKENIZER;
		this.partitionSeparators = PARTITION_SEPARATORS;
		this.concContextSize     = CONC_CONTEXT_SIZE;
		this.typeDefinition      = new Object[] {Tnum.GRAPH};
		this.labels = new PreferencesResourceBundle(locale);
		try {
			this.classTokenizer = Class.forName(DEFAULT_CLASS_TOKENIZER);
		} catch (Exception e) {
			System.err.println("Error "+DEFAULT_CLASS_TOKENIZER+": "+e.getMessage());
			this.classTokenizer = null;
		}
		this.listeners = new EventListenerList();
	}

	/**
	 * Retourne le nom de la police d'affichage.
	 **/
	public String getFontName() {
		return fontName;
	}
	/**
	 * Fixe le nom de la police d'affichage .
	 * @param s     nom de la police
	 **/
	public void setFontName(String s) {
		fontName = s;
		firePreferencesFontChanged();
	}

	/**
	 * Retourne la categorie d'information a mettre dans l'info-bulle
	 **/
	public String getHint() {
		return hint;
	}
	/**
	 * Fixe le nom de la categorie d'information a mettre dans l'info-bulle .
	 * @param s     nom categorie d'information
	 **/
	public void setHint(String s) {
		hint = s;
		firePreferencesHintChanged();
	}

	/**
	 * Retourne la couleur de surlignage .
	 **/
	public Color getBgColor() {
		return bgColor;
	}
	/**
	 * Fixe la couleur de surlignage .
	 * @param col     couleur de surlignage
	 **/
	public void setBgColor(Color col) {
		if (col!=null) {
			bgColor = col;
			firePreferencesBgColorChanged();
		}
	}

	/**
	 * Retourne la locale.
	 **/
	public Locale getLocale() {
		return locale;
	}
	/**
	 * Fixe la locale
	 * @param loc     locale
	 **/
	public void setLocale(Locale loc) {
		locale = loc;
		this.labels = new PreferencesResourceBundle(locale);
		firePreferencesLocaleChanged();
	}

	/**
	 * Fixe la composition du type.
	 **/
	public void setTypeDefinition(Object[] def) {
		typeDefinition = def;
	}
	/**
	 * Retourne la composition du type.
	 **/
	public Object[] getTypeDefinition() {
		return typeDefinition;
	}
	/**
	 * Retourne la locale du tri.
	 **/
	public Locale getLocale4collator() {
		return locale4collator;
	}
	/**
	 * Fixe la locale du tri
	 * @param loc     locale du tri
	 **/
	public void setLocale4collator(Locale loc) {
		locale4collator = loc;
		firePreferencesLocale4collatorChanged();
	}

	/**
	 * Retourne la classe utilisee pour segmenter.
	 * @return segmenteur
	 **/
	public Class getTokenizer() {
		return classTokenizer;
	}
	/**
	 * Fixe la classe a utiliser pour segmenter.
	 * @param c     Class a utiliser pour segmenter
	 **/
	public void setTokenizer(Class c) {
		classTokenizer = c;
		firePreferencesTokenizerChanged();
	}

	/**
	 * Retourne la liste des caracteres separateurs utilises pour decouper un document en segments.
	 * @return liste des caracteres separateurs
	 **/
	public String getStrTokenizer() {
		return strTokenizer;
	}
	/**
	 * Fixe les caracteres separateurs utilises pour decouper un document en segments.
	 * @param str     caracteres separateurs
	 **/
	public void setStrTokenizer(String str) {
		strTokenizer = str;
		firePreferencesStrTokenizerChanged();
	}

	/**
	 * Fixe les caracteres separateurs utilises pour decouper un document en parties.
	 * @param s     caracteres separateurs
	 **/
	public void setPartitionSeparators(String s) {
		partitionSeparators = s;
	}
	/**
	 * Retourne la liste des caracteres separateurs utilises pour decouper un document en parties.
	 * @return liste des caracteres separateurs
	 **/
	public String getPartitionSeparators() {
		return partitionSeparators;
	}
	/**
	 * Retourne le nombre d'unites a retourner dans les contextes droit et gauche des concodances.
	 **/
	public int getContextSize() {
		return concContextSize;
	}
	/**
	 * Fixe le nombre d'unites a retourner dans les contextes droit et gauche des concordances.
	 * @param n     Nombre d'unites
	 **/
	public void setContextSize(int n) {
		concContextSize = n;
	}


	public void firePreferencesFontChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedFontEvent(this, fontName));
		}
	}
	public void firePreferencesHintChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedHintEvent(this, hint));
		}
	}
	public void firePreferencesBgColorChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedBgColorEvent(this, bgColor));
		}
	}
	public void firePreferencesLocale4collatorChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedCollatorEvent(this, locale4collator));
		}
	}
	public void firePreferencesLocaleChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedLocaleEvent(this, labels));
		}
	}
	public void firePreferencesTokenizerChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedTokenizerEvent(this, classTokenizer));
		}
	}
	public void firePreferencesStrTokenizerChanged(){
		PreferencesListener[] listenerList = (PreferencesListener[])listeners.getListeners(PreferencesListener.class);
		for(int i=0; i<listenerList.length; i++) {
			PreferencesListener listener = listenerList[i];
			listener.preferencesChanged(new PreferencesChangedStrTokenizerEvent(this, strTokenizer));
		}
	}
	public void addPreferencesListener(PreferencesListener listener){
		listeners.add(PreferencesListener.class, listener);
	}
	public void removePreferencesListener(PreferencesListener l){
		 listeners.remove(PreferencesListener.class, l);
	}
};
