/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.reader;

import java.io.*;
import java.net.*;
import java.util.zip.*;
import org.xml.sax.*;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.swing.text.SimpleAttributeSet;
/*-----------------------------------------------------------------------*/

/** Lecteur pour le contenu textuel d'un fichier odt.
*/
public class OdtDataReader implements ReaderAccessibility {
		private XMLDocument   doc;

	/**
	 * Cree un lecteur pour un fichier odt.
	 * Ne recupere que le contenu textuel du document.
	 * @param fic Fichier odt a lire
	 **/
	public OdtDataReader(File fic) throws Exception {
		this.doc       = new XMLDocument();
		ZipFile zipFile = new ZipFile(fic);
		ZipEntry zipEntry = zipFile.getEntry("content.xml");
		if (zipEntry!=null) {
			InputStream is = zipFile.getInputStream(zipEntry);
			readIt(is);
		}
	}

	/**
	 * Cree un lecteur pour une url odt.
	 * Ne recupere que le contenu textuel du document.
	 * @param url URL du document odt a lire
	 **/
	public OdtDataReader(URL url) throws Exception {
		this.doc       = new XMLDocument();
		ZipInputStream zis = new ZipInputStream(url.openStream());
		ZipEntry zipEntry = zis.getNextEntry();
		while (zipEntry != null) {
			if (zipEntry.getName().equals("content.xml")) {
				readIt(zis);
				return;
			} else {
				zis.closeEntry();
			}
			zipEntry = zis.getNextEntry();
		}
	}

	/**
	 * Construit un lecteur a partir du flux de donnees
	 **/
	private void readIt(InputStream stream) throws Exception {
		XMLReader reader = XMLReaderFactory.createXMLReader();
		SaxTrameCadreHandler handler = new SaxTrameCadreHandler();
		reader.setContentHandler(handler);
		reader.setErrorHandler(handler);
		reader.parse(new InputSource(stream));
		doc.processBatchUpdates(0);
	}
	/**
	 * Retourne la structure du document
	 **/
	public javax.swing.text.Element getStructure() {
		return doc.getDefaultRootElement();
	}
	/**
	 * Retourne la contenu textuel du document
	 **/
	public String getData() throws Exception {
		return doc.getText(0, doc.getLength());
	}

	/*************************************************************************
	/** Capture des evenements sax.
	**************************************************************************/
	private class SaxTrameCadreHandler extends org.xml.sax.helpers.DefaultHandler {

		public SaxTrameCadreHandler() {
			super();
		}
		//public void ignorableWhitespace(char[] ch, int start, int length) {
		//	characters(ch, start, length);
		//}
		public void characters(char[] ch, int start, int length) {
			String s = new String(ch, start, length);
			try {
        		SimpleAttributeSet atts = new SimpleAttributeSet();
				doc.appendString(s, atts);
			} catch (Exception e) {
				System.err.println("appendString: "+e.getMessage());
			}
		}
		public void startElement(String uri, String localName, String qName, Attributes attributes) {
			try {
        		SimpleAttributeSet atts = new SimpleAttributeSet();
        		atts.addAttribute("tagname", localName);
				doc.appendStartTag(atts);
			} catch (Exception e) {
				System.err.println("appendStartTag: "+e.getMessage());
			}
		}
		public void endElement(String uri, String localName, String qName) {
			try {
				SimpleAttributeSet atts = new SimpleAttributeSet();
				if (localName.equals("p")) {
					doc.appendString("\n", atts);
				} else if (localName.equals("h")) {
					doc.appendString("\n", atts);
				} else if (localName.equals("tab")) {
					doc.appendString("\t", atts);
				} else if (localName.equals("s")) {
					doc.appendString(" ", atts);
				}
        		atts.addAttribute("tagname", localName);
				doc.appendEndTag(atts);
			} catch (Exception e) {
				System.err.println("appendEndTag: "+e.getMessage());
			}
		}
	}
}
