/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2009 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.reader;

import socle.*;
import socle.tokenizer.*;
import socle.lexicon.article.*;

import java.util.*;
import java.io.*;
import java.net.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import javax.xml.transform.sax.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.util.regex.*;

import javax.swing.text.SimpleAttributeSet;
/*-----------------------------------------------------------------------*/

/** Lecteur pour le contenu textuel d'un fichier xml.
*/
public class XMLDataReader implements ReaderAccessibility {
		private XMLDocument          doc;
		private String               input;
		private boolean              localFile;

	/**
	 * Cree un lecteur pour un fichier texte.
	 * @param fic Document xml a lire
	 **/
	public XMLDataReader(File fic) throws Exception {
		this.localFile = true;
		this.input = fic.toString();
		this.doc  = new XMLDocument();
	}

	/**
	 * Cree un lecteur pour un fichier texte.
	 * @param url Document xml a lire
	 **/
	public XMLDataReader(URL url) throws Exception {
		this.localFile = false;
		this.input     = url.toString();
		this.doc       = new XMLDocument();
	}

	/**
	 * Segmente le contenu textuel
	 * @param tokenizer Segmenteur
	 **/
	public void tokenize(Tokenizer tokenizer) throws Exception {
        String stylesheet = getStylesheet();
		if (stylesheet != null) {
			SAXTransformerFactory transformerFactory = (SAXTransformerFactory)TransformerFactory.newInstance();
			if (!transformerFactory.getFeature(SAXTransformerFactory.FEATURE)) {
				throw new RuntimeException("SAX transformation factory not found");
			}
			if (localFile) {
				File fic = new File(input);
				Transformer transformer = transformerFactory.newTransformer(new StreamSource(new File(fic.getParentFile(), stylesheet)));
				SaxTrameCadreHandler myhandler = new SaxTrameCadreHandler(tokenizer);
				transformer.transform(new StreamSource(new FileInputStream(fic)), new SAXResult(myhandler));
			} else {
				URL url = new URL(input);
				String base = url.getProtocol()+"://"+url.getHost()+":"+url.getPort()+"/";
				URL stylesheetUrl = new URL(url.getProtocol(), url.getHost(), url.getPort(), stylesheet);
				Transformer transformer = transformerFactory.newTransformer(new StreamSource(stylesheetUrl.openStream()));
				SaxTrameCadreHandler myhandler = new SaxTrameCadreHandler(tokenizer);
				transformer.transform(new StreamSource(url.openStream()), new SAXResult(myhandler));
			}
		} else {
				XMLReader reader2 = XMLReaderFactory.createXMLReader();
				SaxTrameCadreHandler h = new SaxTrameCadreHandler(tokenizer);
				reader2.setContentHandler(h);
				reader2.setErrorHandler(h);
				if (localFile) {
					File fic = new File(input);
					reader2.parse(new InputSource(new FileInputStream(fic)));
				} else {
					URL url = new URL(input);
					reader2.parse(new InputSource(url.openStream()));
				}
		}
		doc.processBatchUpdates(0);
	}
	private String getStylesheet() {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser parser = factory.newSAXParser();
			XMLReader reader = parser.getXMLReader();

			StylesheetPIHandler handler = new StylesheetPIHandler();
			reader.setContentHandler(handler);
			reader.setErrorHandler(handler);
			if (localFile) {
				File fic = new File(input);
				reader.parse(new InputSource(new FileInputStream(fic)));
			} else {
				URL url = new URL(input);
				reader.parse(new InputSource(url.openStream()));
			}
			return handler.getHref();
		} catch(Exception e) {
			return null;
		}
	}
	/**
	 * Retourne la structure du document
	 **/
	public javax.swing.text.Element getStructure() {
		return doc.getDefaultRootElement();
	}
	/**
	 * Retourne la contenu textuel du document
	 **/
	public String getData() throws Exception {
		return doc.getText(0, doc.getLength());
	}

	/*************************************************************************
	/** Capture des evenements sax.
	**************************************************************************/
	class StylesheetPIHandler extends DefaultHandler {

		private String href = null;

		public String getHref() {
			return href;
		}

		public void processingInstruction(String target, String data) throws SAXException {
			Pattern typePattern = Pattern.compile(".*type\\s*=\\s*\"(\\S*)\".*");
			Pattern hrefPattern = Pattern.compile(".*href\\s*=\\s*\"(\\S*)\".*");
			if (target.equals("xml-stylesheet")) {
				Matcher typeMatcher = typePattern.matcher(data);
				if (typeMatcher.matches() && typeMatcher.group(1).equals("text/xsl")) {
					Matcher hrefMatcher = hrefPattern.matcher(data);
					if (hrefMatcher.matches())
						href = hrefMatcher.group(1);
				}
			}
		}
	}
	/*************************************************************************
	/** Capture des evenements sax.
	**************************************************************************/
	private class SaxTrameCadreHandler extends org.xml.sax.helpers.DefaultHandler {
			Tokenizer                    tokenizer;
			boolean                      isItem;
			String                       itemTag;
			HashMap<String,String>       itemHash;

		public SaxTrameCadreHandler(Tokenizer tokenizer) {
			super();
			this.tokenizer = tokenizer;
			this.isItem = false;
			this.itemHash = new HashMap<String,String>();
			this.itemTag = "";
		}
		void videTampon() {
			if (itemHash.containsKey("f") && itemHash.containsKey("s")) {
				tokenizer.newItem((String)itemHash.get("f"), ((String)itemHash.get("s")).charAt(0));
				itemHash = new HashMap<String,String>();
				itemTag = "";
			}
		}
		public void characters(char[] ch, int start, int length) {
			String s = new String(ch, start, length);
			try {
        		SimpleAttributeSet atts = new SimpleAttributeSet();
				atts.addAttribute("tagname", "text");
				doc.appendString(s, atts);
			} catch (Exception e) {
				System.err.println("appendString: "+e.getMessage());
			}

			try {
				if (isItem) {
					if (!s.equals("")) {
						String o = itemHash.get(itemTag);
						if (o == null) {
							itemHash.put(itemTag, s);
						} else {
							itemHash.put(itemTag, o+s);
						}
					}
				} else {
					String str = s+"\n";
					char[] chars = str.toCharArray();
					CharArrayReader reader = new CharArrayReader(chars, 0, chars.length);
					tokenizer.tokenize(new BufferedReader(reader));
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		}
		public void startElement(String uri, String localName, String qName, Attributes attributes) {
			if (uri.equals(Textometre.TEXTOMETRE_NAMESPACE)) {
				if (localName.equals("item")) {
					isItem = true;
				} else if (isItem) {
					itemTag = localName;
				}
			} else {
				try {
					SimpleAttributeSet atts = new SimpleAttributeSet();
					atts.addAttribute("tagname", localName);
					doc.appendStartTag(atts);
				} catch (Exception e) {
					System.err.println("appendStartTag: "+e.getMessage());
				}
			}
		}
		public void endElement(String uri, String localName, String qName) {
			if (uri.equals(Textometre.TEXTOMETRE_NAMESPACE)) {
				if (localName.equals("item")) {
					videTampon();
					isItem = false;
				}
			} else {
				try {
					SimpleAttributeSet atts = new SimpleAttributeSet();
					atts.addAttribute("tagname", localName);
					doc.appendEndTag(atts);
				} catch (Exception e) {
					System.err.println("appendEndTag: "+e.getMessage());
				}
			}
		}
	}
}
