/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.selection;

import socle.util.*;
import java.io.*;
import java.util.*;
import org.w3c.dom.*;
/*-----------------------------------------------------------------------*/

/** AbstractSelection est une classe abstraite pour representer des selections.
* <p>Une selection est le resultat d'une operation de selection d'objet dans un conteneur d'objets, comme par exemple
* une selection de toutes les occurrences d'une certaine forme dans une trame textuelle.</p>
* Selon la pertinence, ce resultat sera stocke:
* <ul>
* <li>en extension dans un tableau binaire de meme longueur que le le conteneur (par exemple la trame)</li>
* <li>sous forme de positions dans le conteneur</li>
* <li>sous forme d'intervals de positions dans le conteneur</li>
* </ul>
* </p>
*/
public abstract class AbstractSelection implements Serializable {

		//nombre d'occurence de la trame
		protected int                   size;

		//titre de la selection
		private String                  title;

	/** Construit une nouvelle selection vide sans titre.
	 **/
	public AbstractSelection(int size) {
		this("", size);
	}
	/** Construit une nouvelle selection vide avec le titre specifie.
	 **/
	public AbstractSelection(String title, int size) {
		this.size = size;
		setTitle(title);
	}

	/** Construit une nouvelle selection toutes les information codees dans l'element xml.
	 **/
	public AbstractSelection(Element elt) {
		String n = elt.getAttribute("size");
		this.size = (Integer.valueOf(n)).intValue();

		NodeList titles = elt.getElementsByTagName("title");
		Element titleElt = (Element)titles.item(0);
		setTitle(titleElt.getTextContent());
	}

	/** Ajoute un nouvel objet a la selection.
	 **/
	abstract public void add(Object o);

	/** Retourne le resultat de l'union de la selection avec celle specifiee.
	 **/
	abstract public AbstractSelection union(AbstractSelection selection);

	/** Retourne le resultat de l'intersection de la selection avec celle specifiee.
	 **/
	abstract public AbstractSelection intersection(AbstractSelection selection);

	/** Retourne le resultat de la difference de la selection avec celle specifiee.
	 **/
	abstract public AbstractSelection difference(AbstractSelection selection);

	/** Retourne une expression de la selection dans un format xml.
	 **/
	abstract public String toXML();

	/** Retourne un tableau de Range.
	 **/
	abstract public ArrayList<Range> toArray();


	/** Retourne le titre de la selection.
	 **/
	public String getTitle() {
		return title;
	}
	/** Fixe le titre de la selection.
	 **/
	public void setTitle(String title) {
		this.title = title;
	}
	/** Retourne la taille de la trame.
	 **/
	public int getSize() {
		return size;
	}

	/**  Retourne le titre de la selection.
	 **/
	public String toString() {
		return getTitle();
	}
}
