/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.selection;

import socle.util.*;
import java.io.*;
import java.util.*;
/*-----------------------------------------------------------------------*/

/** Representation d'une selection sous forme d'un tableau de booleens.
* Une selection est le resultat d'une operation de selection d'objet dans un conteneur d'objets, comme par exemple
* une selection de toutes les occurrences d'une certaine forme dans une trame textuelle.
* Ce resultat sera stocke sous forme de positions dans le conteneur
*/
public class BooleanSelection extends AbstractSelection {

	private boolean[] set;

	/** Cree une nouvelle selection vide.
	 **/
	public BooleanSelection(int size) {
		super(size);
		this.set = new boolean[size];
		for (int i=0; i<size; i++) {
			set[i] = false;
		}
	}
	/** Cree une nouvelle selection vide.
	 **/
	public BooleanSelection(String title, int size) {
		this(size);
		setTitle(title);
	}
	/** Cree une nouvelle selection.
	 **/
	public BooleanSelection(boolean[] set, int size) {
		super(size);
		this.set = set;
	}

	/** Ajoute un nouvel objet a la selection.
	 **/
	public void add(Object o) {
		if (o instanceof Integer) {
			Integer position = (Integer)o;
			set[position.intValue()] = true;
		} else if (o instanceof Range) {
			Range interval = (Range)o;
			for (int i=interval.getMinValue(); i<=interval.getMaxValue(); i++) {
				set[i] = true;
			}
		} else {
			System.out.println("Unexpected object for BooleanSelection.add(Object o) method");
		}
	}


	/** Retourne le resultat de l'union de la selection avec celle specifiee.
	 **/
	public AbstractSelection union(AbstractSelection selection) {
		boolean[] result = new boolean[size];
		boolean[] set2 = null;

		if (selection instanceof IntervalSelection) {
			IntervalSelection sel_tmp = (IntervalSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel_tmp = (PositionSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection sel_tmp = (BooleanSelection)selection;
			set2 = sel_tmp.set;
		} else {
			System.out.println("Unexpected object for BooleanSelection.union(AbstractSelection selection) method");
		}

		for (int i=0; i<size; i++) {
			result[i] = (set[i] || set2[i]);
		}
		return new BooleanSelection(result, size);
	}

	/** Retourne le resultat de l'intersection de la selection avec celle specifiee.
	 **/
	public AbstractSelection intersection(AbstractSelection selection) {
		boolean[] result = new boolean[size];
		boolean[] set2 = null;

		if (selection instanceof IntervalSelection) {
			IntervalSelection sel_tmp = (IntervalSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel_tmp = (PositionSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection sel_tmp = (BooleanSelection)selection;
			set2 = sel_tmp.set;
		} else {
			System.out.println("Unexpected object for BooleanSelection.intersection(AbstractSelection selection) method");
		}

		for (int i=0; i<size; i++) {
			result[i] = (set[i] && set2[i]);
		}
		return new BooleanSelection(result, size);
	}

	/** Retourne le resultat de la difference de la selection avec celle specifiee.
	 **/
	public AbstractSelection difference(AbstractSelection selection) {
		boolean[] result = new boolean[size];
		boolean[] set2 = null;

		if (selection instanceof IntervalSelection) {
			IntervalSelection sel_tmp = (IntervalSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel_tmp = (PositionSelection)selection;
			BooleanSelection sel = sel_tmp.toBooleanSelection();
			set2 = sel.set;
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection sel_tmp = (BooleanSelection)selection;
			set2 = sel_tmp.set;
		} else {
			System.out.println("Unexpected object for BooleanSelection.intersection(AbstractSelection selection) method");
		}

		for (int i=0; i<size; i++) {
			result[i] = (set[i] && (!set2[i]));
		}
		return new BooleanSelection(result, size);
	}

	public PositionSelection toPositionSelection() {
		HashSet<Integer> result = new HashSet<Integer>();
		for (int i=0; i<size; i++) {
			if (set[i]) {
				result.add(new Integer(i));
			}
		}
		return new PositionSelection(getTitle(), result, size);
	}
	public String toXML() {
		String titleXML = XMLString.toString(getTitle());
		String resultat = "<selection size='"+size+"'>";
		resultat += "<title>"+titleXML+"</title>";
		for (int i=0; i<size; i++) {
			if (set[i]) {
				resultat += "<zone from='"+i+"' until='"+i+"'/>";
			}
		}
		return resultat+"</selection>";
	}
	public ArrayList<Range> toArray() {
		ArrayList<Range> list = new ArrayList<Range>();
		for (int i=0; i<size; i++) {
			if (set[i]) {
				list.add(new Range(i, i));
			}
		}
		return list;
	}
}
