/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.selection;

import socle.util.*;
import java.io.*;
import java.util.*;
import org.w3c.dom.*;
/*-----------------------------------------------------------------------*/

/** Representation d'une selection sous forme d'une liste d'intervalles.
* Une selection est le resultat d'une operation de selection d'objet dans un conteneur d'objets, comme par exemple
* une selection de toutes les occurrences d'une certaine forme dans une trame textuelle.
* Ce resultat sera stocke sous forme de positions dans le conteneur
*/
public class IntervalSelection extends AbstractSelection {

	private ArrayList<Range> set;

	/** Cree une nouvelle selection avec toutes les informations codees dans l'element xml.
	 **/
	public IntervalSelection(Element elt) {
		super(elt);
		this.set = new ArrayList<Range>();
		NodeList zones = elt.getElementsByTagName("zone");
		for (int i=0; i<zones.getLength(); i++) {
			Element zone = (Element)zones.item(i);
			String from = zone.getAttribute("from");
			String until = zone.getAttribute("until");
			Range range = new Range((Integer.valueOf(from)).intValue(), (Integer.valueOf(until)).intValue());
			add(range);
		}
	}
	/** Cree une nouvelle selection vide.
	 **/
	public IntervalSelection(int size) {
		super(size);
		this.set = new ArrayList<Range>();
	}
	/** Cree une nouvelle selection vide.
	 **/
	public IntervalSelection(String title, int size) {
		super(title, size);
		this.set = new ArrayList<Range>();
	}
	/** Cree une nouvelle selection.
	 **/
	public IntervalSelection(ArrayList<Range> set, int size) {
		super(size);
		this.set = set;
	}

	/** Ajoute un nouvel objet a la selection.
	 **/
	public void add(Object o) {
		if (o instanceof Integer) {
			Integer position = (Integer)o;
			int pos = position.intValue();

			for (int i=0; i<set.size(); i++) {
				Range interval = set.get(i);

				//si la position est deja dans la selection
				if (interval.contains(pos)) {
					return;
				}

				 // si la position est avant celle du debut de la selection
				if (pos < interval.getMinValue()) {
					if (pos == (interval.getMinValue() - 1)) {
						// la position touche l'intervalle
						interval.union(new Range(pos, pos));
					} else {
						interval = new Range(pos, pos);
						set.add(i, interval);
					}

					//touche-t-elle l'intervalle precedent?
					if (i > 0) {
						Range precedent = set.get(i-1);
						if (precedent.intersects(interval)) {
							precedent.union(interval);
							set.remove(interval);
						}
					}
					return;
				}
			}
			//si on n'a pas trouve (on ne s'est pas debranche par un return), on ajoute l'element en fin de liste
			Range interval = new Range(pos, pos);
			set.add(interval);
		} else if (o instanceof Range) {
			Range interval = (Range)o;
			set.add(interval);
		} else {
			System.out.println("Unexpected object for IntervalSelection.add(Object o) method");
		}
	}


	/** Retourne le resultat de l'union de la selection avec celle specifiee.
	 **/
	public AbstractSelection union(AbstractSelection selection) {
		if (selection instanceof IntervalSelection) {
			IntervalSelection sel = new IntervalSelection(set, size);
			sel.setTitle(getTitle());
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			for (int i=0; i<tmp_sel.set.size(); i++) {
				Range interval = tmp_sel.set.get(i);
				sel.add(interval);
			}
			return sel;
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel = (PositionSelection)selection;
			return sel.union(this);
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			PositionSelection sel = tmp_sel.toPositionSelection();
			return sel.union(this);
		}
		return this;
	}

	/** Retourne le resultat de l'intersection de la selection avec celle specifiee.
	 **/
	public AbstractSelection intersection(AbstractSelection selection) {
		if (selection instanceof IntervalSelection) {
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			PositionSelection sel = tmp_sel.toPositionSelection();
			return sel.intersection(this);
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel = (PositionSelection)selection;
			return sel.intersection(this);
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			PositionSelection sel = tmp_sel.toPositionSelection();
			return sel.intersection(this);
		}
		return this;
	}

	/** Retourne le resultat de la difference de la selection avec celle specifiee.
	 **/
	public AbstractSelection difference(AbstractSelection selection) {
		if (selection instanceof IntervalSelection) {
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			PositionSelection sel = tmp_sel.toPositionSelection();
			return sel.difference(this);
		} else if (selection instanceof PositionSelection) {
			PositionSelection sel = (PositionSelection)selection;
			return sel.difference(this);
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			PositionSelection sel = tmp_sel.toPositionSelection();
			return sel.difference(this);
		}
		return this;
	}

	public BooleanSelection toBooleanSelection() {
		boolean[] result = new boolean[size];
		for (int i=0; i<size; i++) {
			result[i] = false;
		}
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Range interval = (Range)iterator.next();
			for (int i=interval.getMinValue(); i<=interval.getMaxValue(); i++) {
				result[i] = true;
			}
		}
		return new BooleanSelection(result, size);
	}
	public PositionSelection toPositionSelection() {
		HashSet<Integer> result = new HashSet<Integer>();
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Range interval = (Range)iterator.next();
			for (int i=interval.getMinValue(); i<=interval.getMaxValue(); i++) {
				result.add(new Integer(i));
			}
		}
		return new PositionSelection(getTitle(), result, size);
	}
	public String toXML() {
		String titleXML = XMLString.toString(getTitle());
		String resultat = "<selection size='"+size+"'>";
		resultat += "<title>"+titleXML+"</title>";
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Range interval = (Range)iterator.next();
			resultat += "<zone from='"+interval.getMinValue()+"' until='"+interval.getMaxValue()+"'/>";
		}
		return resultat+"</selection>";
	}
	public ArrayList<Range> toArray() {
		return new ArrayList<Range>(set);
	}
}
