/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2008 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.selection;

import socle.util.*;
import java.io.*;
import java.util.*;
import org.w3c.dom.*;
/*-----------------------------------------------------------------------*/

/** Representation d'une selection sous forme d'une liste de positions.
* Une selection est le resultat d'une operation de selection d'objet dans un conteneur d'objets, comme par exemple
* une selection de toutes les occurrences d'une certaine forme dans une trame textuelle.
* Ce resultat sera stocke sous forme de positions dans le conteneur
*/
public class PositionSelection extends AbstractSelection {

	private HashSet<Integer> set;

	/** Cree une nouvelle selection avec toutes les informations codees dans l'element xml.
	 **/
	public PositionSelection(Element elt) {
		super(elt);
		this.set = new HashSet<Integer>();
		NodeList zones = elt.getElementsByTagName("zone");
		for (int i=0; i<zones.getLength(); i++) {
			Element zone = (Element)zones.item(i);
			String from = zone.getAttribute("from");
			String until = zone.getAttribute("until");
			Range range = new Range((Integer.valueOf(from)).intValue(), (Integer.valueOf(until)).intValue());
			add(range);
		}
	}
	/** Cree une nouvelle selection vide.
	 **/
	public PositionSelection(String title, int size) {
		super(title, size);
		this.set = new HashSet<Integer>();
	}
	/** Cree une nouvelle selection vide.
	 **/
	public PositionSelection(int size) {
		super(size);
		this.set = new HashSet<Integer>();
	}
	/** Cree une nouvelle selection.
	 **/
	public PositionSelection(String title, HashSet<Integer> set, int size) {
		super(title, size);
		this.set = set;
	}

	/** Ajoute un nouvel objet a la selection.
	 **/
	public void add(Object o) {
		if (o instanceof Integer) {
			Integer num = (Integer)o;
			set.add(num);
		} else if (o instanceof Range) {
			Range interval = (Range)o;
			for (int i=interval.getMinValue(); i<=interval.getMaxValue(); i++) {
				set.add(new Integer(i));
			}
		} else {
			System.out.println("Unexpected object for PositionSelection.add(Object o) method");
		}
	}

	/** Retourne le resultat de l'union de la selection avec celle specifiee.
	 **/
	public AbstractSelection union(AbstractSelection selection) {
		HashSet<Integer> set1 = new HashSet<Integer>(set);
		PositionSelection sel = null;
		if (selection instanceof PositionSelection) {
			sel = (PositionSelection)selection;
		} else if (selection instanceof IntervalSelection) {
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			sel = tmp_sel.toPositionSelection();
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			sel = tmp_sel.toPositionSelection();
		}
		if (sel != null) {
			HashSet<Integer> set2 = sel.set;
			set1.addAll(set2);
		}
		return new PositionSelection(getTitle(), set1, size);
	}

	/** Retourne le resultat de l'intersection de la selection avec celle specifiee.
	 **/
	public AbstractSelection intersection(AbstractSelection selection) {
		HashSet<Integer> set1 = new HashSet<Integer>(set);
		PositionSelection sel = null;
		if (selection instanceof PositionSelection) {
			sel = (PositionSelection)selection;
		} else if (selection instanceof IntervalSelection) {
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			sel = tmp_sel.toPositionSelection();
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			sel = tmp_sel.toPositionSelection();
		}
		if (sel != null) {
			HashSet<Integer> set2 = sel.set;
			set1.retainAll(set2);
		}
		return new PositionSelection(getTitle(), set1, size);
	}

	/** Retourne le resultat de la difference de la selection avec celle specifiee.
	 **/
	public AbstractSelection difference(AbstractSelection selection) {
		HashSet<Integer> set1 = new HashSet<Integer>(set);
		PositionSelection sel = null;
		if (selection instanceof PositionSelection) {
			sel = (PositionSelection)selection;
		} else if (selection instanceof IntervalSelection) {
			IntervalSelection tmp_sel = (IntervalSelection)selection;
			sel = tmp_sel.toPositionSelection();
		} else if (selection instanceof BooleanSelection) {
			BooleanSelection tmp_sel = (BooleanSelection)selection;
			sel = tmp_sel.toPositionSelection();
		}
		if (sel != null) {
			HashSet<Integer> set2 = sel.set;
			set1.removeAll(set2);
		}
		return new PositionSelection(getTitle(), set1, size);
	}

	public IntervalSelection toIntervals() {
		HashSet<Range> result = new HashSet<Range>();
		TreeSet<Integer> list = new TreeSet<Integer>(set); //trie l'ensemble pour etre sur de reperer les intervals d'unites qui se suivent
		Iterator iterator = list.iterator();
		int from = -1;
		int until = -1;
		while (iterator.hasNext()) {
			int num = ((Integer)iterator.next()).intValue();
			if (from == -1) {
				from = num;
				until = num;
			} else if (num == (until+1)) {
				until = num;
			} else {
				result.add(new Range(from, until));
				from = num;
				until = num;
			}
		}
		if ((from != -1) && (until != -1)) {
			result.add(new Range(from, until));
		}
		return new IntervalSelection(new ArrayList<Range>(result), size);
	}

	public BooleanSelection toBooleanSelection() {
		boolean[] result = new boolean[size];
		for (int i=0; i<size; i++) {
			result[i] = false;
		}
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Integer num = (Integer)iterator.next();
			result[num.intValue()] = true;
		}
		return new BooleanSelection(result, size);
	}
	public String toXML() {
		String titleXML = XMLString.toString(getTitle());
		String resultat = "<selection size='"+size+"'>";
		resultat += "<title>"+titleXML+"</title>";
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Integer num = (Integer)iterator.next();
			resultat += "<zone from='"+num+"' until='"+num+"'/>";
		}
		return resultat+"</selection>";
	}
	public ArrayList<Range> toArray() {
		ArrayList<Range> list = new ArrayList<Range>();
		Iterator iterator = set.iterator();
		while (iterator.hasNext()) {
			Integer num = (Integer)iterator.next();
			list.add(new Range(num.intValue(), num.intValue()));
		}
		return list;
	}
}
