/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.tokenizer;

import socle.trame.*;
import socle.lexicon.article.*;
import java.io.*;
/*-----------------------------------------------------------------------*/

/** Classe abstraite pour un Segmenteur.
*
* Un segmentateur sait decouper des flux de caracteres en items lexicaux afin d'alimenter une trame textuelle.
*/
public abstract class Tokenizer  {
		private Tokenizable             trame;
		private int                     itemPos;
		private boolean                 feedLexicon;

	/**
	 * Segmenteur.
	 *
	 * @param trame Trame textuelle
	 **/
	public Tokenizer(Tokenizable trame) {
		this.trame    = trame;
		this.itemPos  = -1;
		this.feedLexicon = true;
	}

	/**
	 * flag qui autorise ou nom la mise  jour du lexique.
	 **/
	public void setFeedLexicon(boolean feedLexicon) {
		this.feedLexicon = feedLexicon;
	}
	/**
	 * Retourne le nombre actuel de caracteres.
	 **/
	public int getItemPos() {
		return itemPos;
	}

	/**
	 * Retourne le tokenizer de la classe passee en parametre
	 * @param trame La trame a alimenter
	 * @param classname Nom de la classe tokenizer
	 * @param param Paramtre du tokenizer (liste des separateurs)
	 **/
	public static Tokenizer newInstance(Tokenizable trame, String classname, String param) throws Exception {
		Class classTokenizer = Class.forName(classname);
		return newInstance(trame, classTokenizer, param);
	}
	/**
	 * Retourne le tokenizer de la classe passee en parametre
	 * @param trame La trame a alimenter
	 * @param classTokenizer Classe tokenizer
	 * @param param Paramtre du tokenizer (liste des separateurs)
	 **/
	public static Tokenizer newInstance(Tokenizable trame, Class classTokenizer, String param) throws Exception {
		java.lang.reflect.Constructor[] constructeurs = classTokenizer.getConstructors();
		if (constructeurs.length >= 1) {
			Class[] paramTypes = constructeurs[0].getParameterTypes();
			if (paramTypes.length == 1) {
				java.lang.reflect.Constructor constructeur = classTokenizer.getConstructor (new Class [] {Class.forName("socle.tokenizer.Tokenizable")});
				Tokenizer tokenizer = (Tokenizer)constructeur.newInstance(new Object [] {trame});
				return tokenizer;
			} else if (paramTypes.length == 2) {
				java.lang.reflect.Constructor constructeur = classTokenizer.getConstructor (new Class [] {Class.forName("socle.tokenizer.Tokenizable"), param.getClass()});
				Tokenizer tokenizer = (Tokenizer)constructeur.newInstance(new Object [] {trame, param});
				return tokenizer;
			}
		}
		throw new Exception("Instanciation trouble with Class Tokenizer");
	}

	/**
	 * Ajoute un nouvel item a la trame textuelle.
	 **/
	public void newItem(String item, char stat) {
		itemPos++;
		if (trame!=null) {
			try {
				trame.putArticle(item, stat, feedLexicon);
			} catch (Exception e) {
				System.err.println("Err: "+e.getMessage());
			}
		}
	}
	/**
	 * methode abstraite pour decouper le flux d'un lecteur en items lexicaux.
	 **/
	abstract public void tokenize(BufferedReader reader) throws Exception;
}
