/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.tokenizer;

import socle.trame.*;
import socle.lexicon.article.*;
import java.io.*;
/*-----------------------------------------------------------------------*/

/** Segmenteur en mots utilisant les classes de separateurs
* et de ponctuation d'Unicode.
*/
public class WordTokenizer extends Tokenizer {

	/**
	 * Construit un segmenteur qui decoupe des chaines de caracteres en mots
	 * en utilisant les classes de separateurs et de ponctuation Unicode.
	 *
	 * @param trame Trame textuelle
	 **/
	public WordTokenizer(Tokenizable trame) {
		super(trame);
	}

	/**
	 * Segmente une chaine de caracteres en items.
	 *
	 * <p>Les caracteres des categories Unicode:
	 * <br>Ponctuations:
	 * <ul>
	 * 		<li>Pc: CONNECTOR_PUNCTUATION</li>
	 * 		<li>Pd: DASH_PUNCTUATION</li>
	 * 		<li>Ps: START_PUNCTUATION</li>
	 * 		<li>Pe: END_PUNCTUATION</li>
	 * 		<li>Pi: INITIAL_QUOTE_PUNCTUATION</li>
	 * 		<li>Pf: FINAL_QUOTE_PUNCTUATION</li>
	 * 		<li>Po: OTHER_PUNCTUATION</li>
	 * </ul>
	 * Separateurs:
	 * <ul>
	 * 		<li>Zs: SPACE_SEPARATOR</li>
	 * 		<li>Zl: LINE_SEPARATOR</li>
	 * 		<li>Zp: PARAGRAPH_SEPARATOR</li>
	 * </ul>
	 * Control
	 * <ul>
	 * 		<li>Cc: CONTROL</li>
	 * </ul>
	 * sont etiquettes avec un statut Article.SEPARATOR les autres ont un statut Article.LEXICAL</p>
	 *
	 * @param reader lecteur sur la chaine de caracteres
	 **/
	public void tokenize(BufferedReader reader) throws Exception {
		StringBuffer item = new StringBuffer();
		char c;
		int type;

		String line = reader.readLine();
		while (line != null) {
			for (int pos = 0; pos < line.length(); pos++) {
				c = line.charAt(pos);
				type = Character.getType(c);

				if ((type == Character.CONNECTOR_PUNCTUATION) ||
					(type == Character.DASH_PUNCTUATION) ||
					(type == Character.START_PUNCTUATION) ||
					(type == Character.END_PUNCTUATION) ||
					(type == Character.INITIAL_QUOTE_PUNCTUATION) ||
					(type == Character.FINAL_QUOTE_PUNCTUATION) ||
					(type == Character.OTHER_PUNCTUATION) ||
					(type == Character.SPACE_SEPARATOR) ||
					(type == Character.LINE_SEPARATOR) ||
					(type == Character.PARAGRAPH_SEPARATOR) ||
					(type == Character.CONTROL)) {
					if (item.length() > 0) {
						newItem(item.toString(), Article.LEXICAL);
						item = new StringBuffer();
					}
					newItem(Character.toString(c), Article.SEPARATOR);
				} else {
					item.append(c);
				}
			}
			if (item.length() > 0) {
				newItem(item.toString(), Article.LEXICAL);
				item = new StringBuffer();
			}
			line = reader.readLine();
			if (line!= null) newItem("\n", Article.NEWLINE);
		}
	}

}
