/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.*;
import socle.util.*;
import socle.preferences.*;
import socle.tokenizer.*;
import socle.reader.*;
import socle.lexicon.article.*;

import java.util.*;
import java.io.*;
import java.text.Collator;
import javax.swing.event.EventListenerList;

/*-----------------------------------------------------------------------*/

/** Modele de donnees definissant une trame textuelle.
* <p>Une trame textuelle est le resultat de la segmentation d'un corpus segmente en unites (Article).
*
* Le modele est vide car chaque unite est simplement ecrire en sortie
*/
public class DefaultTrameModel implements Serializable, Tokenizable, PreferencesListener {

		protected String                strTokenizer;
		protected Class                 classTokenizer;
		protected EventListenerList     listeners;
		private OutputStream            stream;

		//ordre alphabetique
		static protected Collator       COLLATOR = null;

		//Comparateur lexicographique (tri primaire stat croissant, tri secondaire ordre alpha croissant
		public static final Comparator<String> GRAPH_ORDER = new Comparator<String>() {
			public int compare(String obj1, String obj2) {
				if (COLLATOR != null) {
					return COLLATOR.compare(obj1, obj2);
				} else {
					return obj1.compareTo(obj2);
				}
			}
		};
		//Comparateur lexicographique (tri primaire stat croissant, tri secondaire ordre alpha croissant
		public static final Comparator<Article> LEXICOG_ORDER = new Comparator<Article>() {
			public int compare(Article obj1, Article obj2) {				//int stat1 = obj1.getStat();
				//int stat2 = obj2.getStat();
				//if (stat1 == stat2) {
					if (COLLATOR != null) {
						return COLLATOR.compare(obj1.getGraph(), obj2.getGraph());
					} else {
						return obj1.getGraph().compareTo(obj2.getGraph());
					}
				//} else {
				//	return (stat1 < stat2) ? -1 : 1;
				//}
			}
		};
		//Comparateur lexicometrique (tri primaire stat croissant, tri secondaire freq croissant, tri tertiaire ordre alpha croissant)
		public static final Comparator<Article> LEXICOM_ORDER = new Comparator<Article>() {
			public int compare(Article obj1, Article obj2) {
				//int stat1 = obj1.getStat();
				//int stat2 = obj2.getStat();
				//if (stat1 == stat2) {
					if (obj1.getFreq() == obj2.getFreq()) {
						if (COLLATOR != null) {
							return COLLATOR.compare(obj1.getGraph(), obj2.getGraph());
						} else {
							return obj1.getGraph().compareTo(obj2.getGraph());
						}
					} else if (obj1.getFreq() < obj2.getFreq()) {
						return 1;
					} else {
						return -1;
					}
				//} else {
				//	return (stat1 < stat2) ? -1 : 1;
				//}
			}
		};

	/**
	 * Construit une nouvelle trame qui ecrit directement dans la sortie standard.
	 **/
	public DefaultTrameModel() {
		this(System.out);
	}
	/**
	 * Construit une nouvelle trame qui ecrit dans le stream specifie.
	 **/
	public DefaultTrameModel(OutputStream stream) {
		this.listeners = new EventListenerList();
		this.stream = stream;
		this.strTokenizer = PreferencesModel.STR_TOKENIZER;
		try {
			this.classTokenizer = Class.forName(PreferencesModel.DEFAULT_CLASS_TOKENIZER);
		} catch (Exception e) {
			System.err.println("Error "+PreferencesModel.DEFAULT_CLASS_TOKENIZER+": "+e.getMessage());
			this.classTokenizer = null;
		}
	}

	/**
	 * Retourne un tokenizer pour la trame
	 **/
	public Tokenizer getNewTokenizer() {
		try {
			return Tokenizer.newInstance(this, classTokenizer, strTokenizer);
		} catch (Exception e) {
			System.err.println("Instanciation trouble with Class Tokenizer: "+e.getMessage());
			return null;
		}
	}

	/**
	 * Ajoute un TrameListener a la trame
	 **/
	public void addTrameListener(TrameListener listener){
		listeners.add(TrameListener.class, listener);
	}

	/**
	 * Supprime le TrameListener de la trame
	 **/
	public void removeTrameListener(TrameListener l){
		 listeners.remove(TrameListener.class, l);
	}


	/***********************************************************
	 * implementation des methodes d'interface Tokenizable
	 ***********************************************************/

	/** Ajoute un article dans le lexique ou augmente sa frequence.
	 * @param graph     Representation graphique de l'article.
	 * @param stat      Statut de l'article (Article.LEXICAL, Article.SEPARATOR).
	 *
	 * @return Le numero d'ordre de l'article
	 **/
	public void putArticle(String graph, char stat, boolean feedLexicon) throws Exception {
		StringBuffer xmldata = XMLString.toStringBuffer(graph);
		if (stat == Article.NEWLINE) {
			xmldata.append('\n');
		}
		String result = "<item><f>"+xmldata.toString()+"</f><s>"+stat+"</s></item>";
		byte[] content = result.getBytes("UTF-8");
		stream.write(content, 0, content.length);
	}
	/**
	 * Retourne l'article correspondant du lexique.
	 * Cette implementation genere simplement une exception
	 **/
	public Article getArticle(int numapp) throws Exception {
		throw new Exception("No lexicon");
	}
	/**
	 * Retourne l'article correspondant du lexique.
	 * Cette implementation genere simplement une exception
	 **/
	public Article getArticle(String s) throws Exception {
		throw new Exception("No lexicon");
	}

	/**
	 * Ecoute les evenements de modification dans les preferences (Choix du tokenizer et de l'ordre alphabetique)
	 **/
	public void preferencesChanged(PreferencesChangedEvent event) {
		if(event instanceof PreferencesChangedCollatorEvent) {
			PreferencesChangedCollatorEvent e = (PreferencesChangedCollatorEvent)event;
			COLLATOR = Collator.getInstance(e.getNewLocale());
		} else if(event instanceof PreferencesChangedTokenizerEvent) {
			PreferencesChangedTokenizerEvent e = (PreferencesChangedTokenizerEvent)event;
			classTokenizer = e.getNewTokenizer();
		} else if(event instanceof PreferencesChangedStrTokenizerEvent) {
			PreferencesChangedStrTokenizerEvent e = (PreferencesChangedStrTokenizerEvent)event;
			strTokenizer = e.getNewStrTokenizer();
		}
	}
}