/* ***** BEGIN LICENSE BLOCK *****
 *    Copyright 2007 Michel Jacobson michel.jacobson@gmail.com
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * ***** END LICENSE BLOCK ***** */

/*-----------------------------------------------------------------------*/
package socle.trame;

import socle.*;
import socle.util.*;
import socle.selection.*;
import socle.tokenizer.*;
import socle.reader.*;
import socle.preferences.*;
import socle.lexicon.*;
import socle.lexicon.article.*;

import java.util.*;
import java.io.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import javax.xml.transform.sax.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
/*-----------------------------------------------------------------------*/

/** Modele de donnees definissant une trame textuelle.
* <p>Une trame textuelle est le resultat de la segmentation d'un corpus segmente en unites (Article).
* La trame entretient en memoire un lexique <code>LexController</code> de toute les unites distinctes, une forme graphique
* du texte, l'ordre des unites et une liste des positions de depart (en nombre de caracteres) de chaque unite</p>
*
* Le modele est une liste de chiffre
* La vue peut tre la reprsentation typographique de chaque unite, des cubes, ...
*
* <p>Une unite est representee dans la trame par un numero d'ordre correspondant a la position de sa premiere apparition
* dans le corpus.</p>
*/
public class TrameModel extends DefaultTrameModel {
		private LexController         lexController;
		private HashMap<String, Tnum> tnumHash;
		private Facets                facets;

		//Nom du tnum qui est affiche dans la trame
		private String                current_tnum_name;
		private Tnum                  current_tnum;
		private ArrayList<Integer>    current_positions;

	/**
	 * Construit une nouvelle trame en utilisant le lexique en parametre.
	 **/
	public TrameModel(LexController lexController) {
		super();
		this.current_tnum_name = Tnum.GRAPH;
		this.tnumHash = new HashMap<String, Tnum>();
		this.facets   = new Facets();

		addTnumName(current_tnum_name);
		addTnumName(Tnum.STATUS);
		this.current_tnum = getTnum(current_tnum_name);
		this.current_positions = new ArrayList<Integer>();
		this.lexController = lexController;
	}

	/**
	 * Vide la trame.
	 **/
	public void reset() {
		this.tnumHash.clear();
		this.facets.clear();

		fireTrameFieldsChanged(new ArrayList<String>(getTnumNames()));
		addTnumName(Tnum.GRAPH);
		addTnumName(Tnum.STATUS);
		this.current_tnum = getTnum(current_tnum_name);
		this.current_positions.clear();
	}

	/**
	 * Retourne le nom de la vue de la trame courante.
	 **/
	public String getCurrentTrameName() {
		return current_tnum_name;
	}

	/**
	 * Retourne le nom de la vue du lexique courant.
	 **/
	public String getCurrentLexName() {
		return lexController.getName();
	}

	/**
	 * Cree un nouveau tnum du nom specifie.
	 **/
	private void addTnumName(String name) {
		Tnum result = new Tnum();
		setTnum(name, result);
	}
	/**
	 * Retourne le tnum du nom specifie.
	 **/
	public Tnum getTnum(String name) {
		return tnumHash.get(name);
	}
	/**
	 * Fixe le tnum du nom specifie.
	 **/
	public void setTnum(String name, Tnum atnum) {
		tnumHash.put(name, atnum);
		if (name.equals(Tnum.GRAPH)) {
			facets.addFacet(PreferencesModel.DEFAULT_FACET);
		} else {
			facets.addFacet(name);
		}
		fireTrameFieldsChanged(new ArrayList<String>(getTnumNames()));
	}
	private Set<String> getTnumNames() {
		return tnumHash.keySet();
	}
	/**
	 * Retourne l'ensemble des noms des proprietes des items.
	 **/
	public String[] getPropertyNames() {
		Object[] keys = tnumHash.keySet().toArray();
		String[] result = new String[keys.length];
		for (int i=0; i<keys.length; i++) {
			result[i] = (String)keys[i];
		}
		return result;
	}
	/**
	 * Retourne les facettes;.
	 **/
	public Facets getFacets() {
		return facets;
	}
	/**
	 * Fixe un clone du lexique lie au tnum.
	 **/
	private void copyLexicon(Tnum t) {
		current_tnum.copyLexicon(t);
	}

	/**
	 * Charge le lexique correspondant au tnum du nom specifie si il existe sinon retourne false.
	 **/
	public boolean setLexModel(String name) {
		if (tnumHash.containsKey(name)) {
			lexController.setModel(getTnum(name));
			return true;
		}
		return false;
	}

	/**
	 * Charge le tnum du nom specifie si il existe sinon retourne false.
	 **/
	public boolean setCurrentTnum(String name) {
		if (tnumHash.containsKey(name)) {
			current_tnum_name = name;
			current_tnum = getTnum(current_tnum_name);
			return true;
		}
		return false;
	}

//**********************************************************************
//******* Recherches d'occurrences
//**********************************************************************
	/**
	 * Retourne une selection avec toutes les occurrences d'une unit, ou d'une suite d'units dans le trame, .
	 * @param obj     un article, une forme graphique, une zone (du texte ou de la trame), une numero de premiere apparition ou une sous-trame
	 **/
	public AbstractSelection getOccs(Object obj) {
		ZoneTrame zone = new ZoneTrame(0, size()-1);
		return getOccs(obj, zone);
	}
	/**
	 * Retourne une selection avec toutes les occurrences d'une unit, ou d'une suite d'units dans le trame, .
	 * @param obj     un article, une forme graphique, une zone (du texte ou de la trame), une numero de premiere apparition ou une sous-trame
	 * @param zone        la recherche d'occurrences ne s'effectue que dans cette zone de la trame
	 **/
	public AbstractSelection getOccs(Object obj, ZoneTrame zone) {
		if (obj instanceof AbstractSelection) {
			AbstractSelection sel = (AbstractSelection)obj;
			return sel;//attention il n'y a pas le filtre sur la zone
		} else if (obj instanceof Article) {
			int numap = ((Article)obj).getNumap();
			return getOccs(numap, zone);
		} else if (obj instanceof Integer) {
			int numap = ((Integer)obj).intValue();
			return getOccs(numap, zone);
		} else if (obj instanceof String) {
			TrameModel subtrame = makeTrame((String)obj);
			return getOccs(subtrame, zone);
		} else if (obj instanceof ZoneText) {
			ZoneTrame zt = toZoneTrame((ZoneText)obj);
			TrameModel subtrame = subtrame(zt);
			return getOccs(subtrame, zone);
		} else if (obj instanceof ZoneTrame) {
			TrameModel subtrame = subtrame((ZoneTrame)obj);
			return getOccs(subtrame, zone);
		} else if (obj instanceof TrameModel) {
			return getOccs((TrameModel)obj, zone);
		} else {
			System.err.println("Unexpected object class for getOccs(Object obj, ZoneTrame zone)");
			return new PositionSelection(size());
		}
	}
	/**
	 * Retourne une selection avec toutes les occurrences d'une unite.
	 * @param numap       numero de premiere appartition de l'unite
	 * @param zone        la recherche d'occurrences ne s'effectue que dans cette zone de la trame
	 **/
	public AbstractSelection getOccs(int numap, ZoneTrame zone) {
		PositionSelection selection = new PositionSelection(size());
		for (int pos=zone.getMinValue(); pos<=zone.getMaxValue(); pos++) {
			if (getNumap(pos, lexController.getName()) == numap) {
				selection.add(new Range(pos, pos));
			}
		}
		return selection;
	}
	/**
	 * Retourne une selection avec toutes les occurences de l'unite ou la suite d'unites de la sous-trame.
	 * @param subtrame     sous-trame a rechercher dans la trame du texte
	 * @param zone        la recherche d'occurrences ne s'effectue que dans cette zone de la trame
	 **/
	public AbstractSelection getOccs(TrameModel subtrame, ZoneTrame zone) {
		IntervalSelection selection = new IntervalSelection(size());
		int from = zone.getMinValue();
		int pos = indexOf(subtrame, from);
		while ((pos != -1) && (pos<=zone.getMaxValue())) {
			selection.add(new Range(pos, pos+subtrame.size()-1));
			from = pos + 1;
			pos = indexOf(subtrame, from);
		}
		return selection;
	}

	//**********************************************************************
	//******* fonctions get et set
	//**********************************************************************

	/**
	 * Retourne le numero d'ordre de la nieme unite de la trame.
	 * @param numocc      position de l'occurrence dans la trame.
	 **/
	public int getNumap(int numocc) {
		return current_tnum.getInt(numocc);
	}

	/**
	 * Retourne le numero d'ordre de la nieme unite de la trame.
	 * @param numocc      position de l'occurrence dans la trame.
	 * @param name        Nom du tnu dans lequel la recherche est faite.
	 **/
	public int getNumap(int numocc, String name) {
		return getTnum(name).getInt(numocc);
	}
	/**
	 * Retourne la position de depart dans le texte de la nieme unite de la trame.
	 * [position telle que la fonction getSelectionStart des editeurs renvoie]
	 * @param numocc      position dans la trame.
	 **/
	public int getStartPosition(int numocc) {
		if (numocc >= size()) return getEndPosition(size()-1);
		return current_positions.get(numocc).intValue();
	}

	/**
	 * Retourne la position de fin dans le texte de la nieme unite de la trame.
	 * [position telle que la fonction getSelectionEnd des editeurs renvoie]
	 * @param numocc      position dans la trame.
	 **/
	public int getEndPosition(int numocc) {
		int pos = current_positions.get(numocc).intValue();
		String str = (String)current_tnum.getString(numocc);
		pos += str.length();
		return pos;
	}

	/**
	 * Retourne la taille de la trame (nombre d'unites qu'elle comporte).
	 **/
	public int size() {
		return current_tnum.size();
	}

	/**
	 * Teste si la trame est vide d'unite.
	 **/
	public boolean isEmpty() {
		return current_tnum.isEmpty();
	}

	/**
	 * Retourne la zone de la trame en coordonnees du texte
	 **/
	public ZoneText toZoneText(ZoneTrame zone) {
		return new ZoneText(getStartPosition(zone.getMinValue()), getEndPosition(zone.getMaxValue()));
	}

	/**
	 * Retourne la plus petite zone de la trame qui couvre la zone du texte
	 **/
	public ZoneTrame toZoneTrame(ZoneText zone) {
		int from = zone.getMinValue();
		int until = zone.getMaxValue();
		int posStart = 0;
		int posEnd   = 0;
		for (int i=0; i<size(); i++) {
			int s = getStartPosition(i);
			if (from >= s) {
				int f = getEndPosition(i);
				if(from < f) {
					posStart = i;
					posEnd   = i;
					break;
				}
			}
		}
		for (int i=posStart; i<size(); i++) {
			int f = getEndPosition(i);
			if (f >= until) {
				posEnd   = i;
				return new ZoneTrame(posStart, posEnd);
			} else {
				posEnd   = i;
			}
		}
		return new ZoneTrame(posStart, posEnd);
	}

	/**
	 * Retourne la representation graphique d'une portion de la trame.
	 **/
	public String getString(Range range, String tnumName) {
		Tnum atnum = getTnum(tnumName);
		return atnum.getString(range);
	}

	/**
	 * Recalcule les positions de chaque unite dans la representation graphique et retourne celle-ci.
	 **/
	private String rebuild() {
		int currentPosition = 0;
		current_positions.clear();
		StringBuffer content = new StringBuffer(100000);
		for (int i=0; i<size(); i++) {
			String graph = current_tnum.getString(i);
			current_positions.add(new Integer(currentPosition));
			currentPosition +=graph.length();
			content.append(graph);
		}
		return content.toString();
	}

	/**
	 * Retourne une nouvelle trame qui est sous-trame de la trame.
	 * @param zone    Interval a extraire en termes de positions dans la trame.
	 *
	 * @return la sous-trame ou null<ul>
	 * 		<li>si la position de la premiere unite est inferieure a 0 ou superieure a la taille de la trame</li>
	 * 		<li>si la position de la derniere unite est superieure a la taille de la trame
	 * ou inferieure a la position de depart.</li>
	 *</ul>
	 **/
	public TrameModel subtrame(ZoneTrame zone) {
		int indexStart = zone.getMinValue();
		int indexEnd = zone.getMaxValue();
		TrameModel extrait = new TrameModel(lexController);
		extrait.copyLexicon(current_tnum);
		if ((indexStart < 0) || (indexStart >= size()) || (indexEnd < indexStart) || (indexEnd >= size())) return null;
		for (int i=indexStart; i<=indexEnd; i++) {
			try {
				extrait.putArticle(current_tnum.getString(i), Article.LEXICAL, false);
			} catch (Exception e) {
				System.err.println("err "+e.getMessage());
			}
		}
		return extrait;
	}
	/**
	 * Retourne la position de depart de la premiere unite d'une sous-trame en partant d'une position donnee dans la trame.
	 * @param sublist      sous-trame recherchee.
	 * @param indexFrom    position de depart dans la trame.
	 **/
	private int indexOf(TrameModel sublist, int indexFrom) {
		if(!sublist.isEmpty()){
			int posSublistMax = sublist.size();
			int posSublist = 0;
			int numapp = sublist.getNumap(posSublist);

			for (int i=indexFrom; i<size(); i++) {
				int n = getNumap(i);
				if (numapp == n) {
					posSublist++;
					if (posSublist>=posSublistMax) {
						return i - posSublistMax + 1;
					} else {
						numapp = sublist.getNumap(posSublist);
					}
				} else {
					posSublist = 0;
					numapp = sublist.getNumap(posSublist);
				}
			}
		}
		return -1;
	}

	/**
	 * Cree une trame a partir de la chaine passe en parametre.
	 * Utilise le lexique general mais ne l'enrichi pas.
	 **/
	private TrameModel makeTrame(String str) {
		TrameModel subtrame = new TrameModel(lexController);
		subtrame.copyLexicon(current_tnum);
		try {
			Tokenizer tokenizer = Tokenizer.newInstance(subtrame, classTokenizer, strTokenizer);
			tokenizer.setFeedLexicon(false);
			BufferedReader dataReader = new BufferedReader(new StringReader(str));
			tokenizer.tokenize(dataReader);
			dataReader.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return subtrame;
	}

	/**
	 * Lit une trame textuelle a partir de sa forme xml dans un fichier
	 **/
	public void read(File fic) throws Exception {
		XMLReader reader = XMLReaderFactory.createXMLReader();
		mySaxHandler h = new mySaxHandler();
		reader.setContentHandler(h);
		reader.setErrorHandler(h);
		reset();
		reader.parse(new InputSource(new FileInputStream(fic)));
		fireLexChanged();
		current_tnum = getTnum(current_tnum_name);
		fireTrameChanged();
	}
	/**
	 * Ecrit la trame textuelle sous forme xml dans un fichier
	 **/
	public void write(File fic) throws Exception {
		OutputStream stream    = new FileOutputStream(fic);
		String result = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + "<trame xmlns=\""+Textometre.TEXTOMETRE_NAMESPACE+"\">\n";
		byte[] entete = result.getBytes("UTF-8");
		stream.write(entete, 0, entete.length);

		write_dicos(stream);
		write_data(stream);

		result = "</trame>";
		byte[] fin = result.getBytes("UTF-8");
		stream.write(fin, 0, fin.length);
		stream.close();
	}
	private void write_dicos(OutputStream stream) throws Exception {
		String result = "";
		Iterator iterator = getTnumNames().iterator();
		while (iterator.hasNext()) {
			String name = (String)iterator.next();
			Tnum atnum = getTnum(name);

			result = "<dico>\n<name>"+XMLString.toString(name)+"</name>\n";
			byte[] the_content = result.getBytes("UTF-8");
			stream.write(the_content, 0, the_content.length);

			result = atnum.getDico();
			the_content = result.getBytes("UTF-8");
			stream.write(the_content, 0, the_content.length);

			result = "</dico>\n";
			the_content = result.getBytes("UTF-8");
			stream.write(the_content, 0, the_content.length);
		}
	}
	private void write_data(OutputStream stream) throws Exception {
		String result = "<data>\n";
		byte[] the_content = result.getBytes("UTF-8");
		stream.write(the_content, 0, the_content.length);

		Iterator iterator = getTnumNames().iterator();
		List<Tnum> liste = new ArrayList<Tnum>();
		StringBuffer entete = new StringBuffer();
		while (iterator.hasNext()) {
			String name = (String)iterator.next();
			entete.append(name);
			entete.append(iterator.hasNext()?'\t': '\n');
			Tnum atnum = getTnum(name);
			liste.add(atnum);
		}
		the_content = entete.toString().getBytes("UTF-8");
		stream.write(the_content, 0, the_content.length);

		for (int i=0; i<current_tnum.size(); i++) {
			StringBuffer line = new StringBuffer();
			for (int j=0; j<liste.size(); j++) {
				line.append(liste.get(j).getInt(i));
				line.append((j<liste.size()-1)?'\t': '\n');
			}
			the_content = line.toString().getBytes("UTF-8");
			stream.write(the_content, 0, the_content.length);
		}
		result = "</data>\n";
		the_content = result.getBytes("UTF-8");
		stream.write(the_content, 0, the_content.length);
	}
	private void read_data(String str) {
		StringTokenizer lines = new StringTokenizer(str, "\n");
		String header = lines.nextToken();
		StringTokenizer header_fields = new StringTokenizer(header, "\t");
		Tnum[] tnums = new Tnum[header_fields.countTokens()];
		int n = 0;
		while (header_fields.hasMoreTokens()) {
			String header_field = header_fields.nextToken();
			tnums[n] = getTnum(header_field);
			n++;
		}

		while (lines.hasMoreTokens()) {
			String line = lines.nextToken();
			StringTokenizer fields = new StringTokenizer(line, "\t");
			int pos = 0;
			while (fields.hasMoreTokens()) {
				String field = fields.nextToken();
				tnums[pos].superadd(new Integer(field));
				pos++;
			}
		}
	}

	//**********************************************************************
	//******* modifications dans la trame
	//**********************************************************************

	/**
	 * Change la forme de toutes les occurences de la selection par la nouvelle valeur.
	 * @param sel            La selection de toutes les positions qui doivent etre changees.
	 * @param new_graph      Nouvelle valeur des occurrences.
	 * @param lex_name       Nom du Tnum qui doit etre modifie.
	 **/
	public void changeAllOccurences(AbstractSelection sel, String new_graph, String lex_name) {
		ArrayList tableau = sel.toArray();
		for (int i=0; i<tableau.size(); i++) {
			Range range = (Range)tableau.get(i);
			for (int numocc=range.getMinValue(); numocc<=range.getMaxValue(); numocc++) {
				getTnum(lex_name).changeOcc(numocc, new_graph);
			}
		}
	}

	/**
	 * Change les proprietes d'une occurrence par celles d'une autre.
	 * @param old_occ      Ancienne occurence.
	 * @param new_occ      Nouvelle occurence.
	 **/
	public void changeOccurence(OccurrenceModel old_occ, OccurrenceModel new_occ) {
		int numocc = old_occ.getNumocc();

		Iterator iterator = getTnumNames().iterator();
		while (iterator.hasNext()) {
			String name = (String)iterator.next();
			Tnum atnum = getTnum(name);
			String old_str = old_occ.getValue(name);
			String new_str = new_occ.getValue(name);
			if (!new_str.equals(old_str)) {

				//mise a jour des tnum
				atnum.changeOcc(numocc, new_str);

				//mise a jour des positions pour la vue courante
				if (name.equals(current_tnum_name)) {
					int diff = old_str.length() - new_str.length();
					if (diff != 0) shiftStartPositions(numocc+1, diff);
				}

			}
		}

	}

	//il faut aussi changer la definition des zones dans les structures (dans les partitions bof)
	/**
	 * Fusionne une suite d'occurrences en une seule et lui donne la valeur specifiee.
	 **/
	public void replaceOccsByOcc(ArrayList<OccurrenceModel> old_occs, OccurrenceModel new_occ, int numocc) {
		int startPos = getStartPosition(numocc);

		//mise a jour des tnums (supprime les anciennes occurences)
		String str = "";
		for (int i=0; i<old_occs.size(); i++) {
			OccurrenceModel occ = old_occs.get(i);
			Iterator iterator = getTnumNames().iterator();
			while (iterator.hasNext()) {
				String name = (String)iterator.next();
				if (name.equals(current_tnum_name)) str += (String)occ.getValue(name);
				Tnum atnum = getTnum(name);
				atnum.remove(numocc);
			}
		}

		//mise a jour des tnums (insere la nouvelle occurence)
		Iterator iterator = getTnumNames().iterator();
		while (iterator.hasNext()) {
			String name = (String)iterator.next();
			Tnum atnum = getTnum(name);
			atnum.add(numocc, new_occ.getValue(name));
		}

		//mise a jour des positions pour la vue courante
		for (int i=0; i<old_occs.size(); i++) {
			current_positions.remove(numocc);
		}
		current_positions.add(numocc, new Integer(startPos));
		int diff = str.length() - new_occ.getValue(current_tnum_name).length();
		if (diff != 0) shiftStartPositions(numocc+1, diff);
	}

	//il faut aussi changer la definition des zones dans les structures (dans les partitions bof)
	/**
	 * Remplace une occurrence par une liste d'occurrences.
	 * @param old_occ      Occurence a supprimer
	 * @param list_occs    Liste ordonnee des occurences a inserer
	 * @param numocc       Point d'insertion
	 **/
	public void replaceOccByOccs(OccurrenceModel old_occ, ArrayList<OccurrenceModel> list_occs, int numocc) {
		int startPos = getStartPosition(numocc);

		//mise a jour des tnums (supprime l'ancienne occurence et insere les nouvelles)
		current_positions.remove(numocc);
		Iterator iterator = getTnumNames().iterator();
		while (iterator.hasNext()) {
			String name = (String)iterator.next();
			getTnum(name).remove(numocc);
		}
		int startPt = startPos;
		String str = "";
		for (int i=0; i<list_occs.size(); i++) {
			OccurrenceModel occ = list_occs.get(i);
			iterator = getTnumNames().iterator();
			while (iterator.hasNext()) {
				String name = (String)iterator.next();
				Tnum atnum = getTnum(name);
				atnum.add(numocc + i, occ.getValue(name));
			}
			String new_graph = occ.getValue(current_tnum_name);
			current_positions.add(numocc + i, new Integer(startPt));
			str += new_graph;
			startPt += new_graph.length();
		}

		//mise a jour des positions
		int diff = old_occ.getValue(current_tnum_name).length() - str.length();
		if (diff != 0) shiftStartPositions(numocc+list_occs.size()+1, diff);
	}

	/**
	 * Decale toute les positions de debut d'unites de la longueur specifiee.
	 * @param from     position de la premiere unite a decaler
	 * @param to       position de la derniere unite a decaler
	 * @param shift    valeur du decalage
	 **/
	private void shiftStartPositions(int from, int to, int shift) {
		if(shift != 0) {
			for (int i=from; i<=to; i++) {
				int old_val = current_positions.get(i).intValue();
				int new_val = old_val - shift;
				current_positions.set(i, new Integer(new_val));
			}
		}
	}
	/**
	 * Decale toute les positions de debut d'unites de la longueur specifiee.
	 * @param from     position de la premiere unite a decaler
	 * @param shift    valeur du decalage
	 **/
	private void shiftStartPositions(int from, int shift) {
		shiftStartPositions(from, current_positions.size() -1, shift);
	}



	public ArrayList<Integer> getPositions() {
		ArrayList<Integer> positions = new ArrayList<Integer>();
		for (int i=0; i<size(); i++) {
			Integer NUM = new Integer(i);
			try{
				OccurrenceModel occ = getOccurrence(i);
				String graph = occ.getGraph();
				for (int pos=0; pos<graph.length(); pos++) {
					positions.add(NUM);
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		}
		return positions;
	}

	/**
	 * Provoque la mise a jour de l'editeur de la trame.
	 **/
	public void fireTrameChanged() {
		TrameListener[] listenerList = (TrameListener[])listeners.getListeners(TrameListener.class);
		TrameChangedEvent event = new TrameChangedEvent(this, rebuild());
		for(int i=0; i<listenerList.length; i++) {
			TrameListener listener = listenerList[i];
			listener.trameChanged(event);
		}
	}
	public void fireTrameFieldsChanged(ArrayList<String> names) {
		TrameListener[] listenerList = (TrameListener[])listeners.getListeners(TrameListener.class);
		for(int i=0; i<listenerList.length; i++) {
			TrameListener listener = listenerList[i];
			listener.trameChanged(new TrameFieldsChangedEvent(this, names));
		}
	}
	/**
	 * Provoque la mise a jour de l'editeur de lexique.
	 **/
	public void fireLexChanged() {
		String name = getCurrentLexName();
		lexController.setModel(getTnum(name));
	}

	public OccurrenceModel getOccurrence(int numocc) throws Exception {
		String graph = current_tnum.getString(numocc);
		int numap = current_tnum.getInt(numocc);
		HashMap<String,String> properties = new HashMap<String,String>();
		Iterator iterator = getTnumNames().iterator();
		while (iterator.hasNext()) {
			String propertieName = (String)iterator.next();
			String propertieValue = getTnum(propertieName).getString(numocc);
			properties.put(propertieName, propertieValue);
		}
		return new OccurrenceModel(graph, numap, numocc, properties);
	}

	//***********************************************************
	//******* implementation des methodes d'interface Tokenizable
	//***********************************************************

	/** Ajoute un article dans le lexique ou augmente sa frequence.
	 * @param graph     Representation graphique de l'article.
	 * @param stat      Statut de l'article (Article.LEXICAL, Article.SEPARATOR).
	 **/
	public void putArticle(String graph, char stat, boolean feedLexicon) throws Exception {
		getTnum(Tnum.GRAPH).add(new String(graph));
		getTnum(Tnum.STATUS).add(String.valueOf(stat));
	}

	/**
	 * Retourne l'article correspondant au numero d'ordre.
	 * @param numapp     Numero d'ordre
	 *
	 * @return L'article du lexique
	 * @throws Exception si l'article n'est pas trouve dans le lexique
	 **/
	public Article getArticle(int numapp) throws Exception {
		Article art = lexController.getArticle(numapp);
		return art;
	}
	/**
	 * Retourne l'article correspondant a la forme graphique.
	 * @param graph     Representation graphique de l'article
	 *
	 * @return L'article du lexique
	 * @throws Exception si l'article n'est pas trouve dans le lexique
	 **/
	public Article getArticle(String graph) throws Exception {
		Article art = lexController.getArticle(graph);
		return art;
	}
























	/*************************************************************************
	/** Capture des evenements sax.
	**************************************************************************/
	private class mySaxHandler extends org.xml.sax.helpers.DefaultHandler {

		private boolean entry_bool, data_bool, tnum_bool, name_bool, f_bool, freq_bool, id_bool;
		private Integer key;
		private Tnum atnum;
		private HashMap<Integer, Integer>              mapFreq;
		private HashMap<Object, Integer>               properties;
		private HashMap<Integer, Object>               invert;
		private StringBuffer data;
		private String forme;

		public mySaxHandler() {
			super();
			tnum_bool = false;
			name_bool = false;
			entry_bool = false;
			data_bool = false;
			f_bool = false;
			freq_bool = false;
			id_bool = false;

			atnum = null;
			key = null;
			mapFreq =  null;
			properties = null;
			invert =  null;
			data = new StringBuffer();
			forme = "";
		}
		public void characters(char[] ch, int start, int length) {
			String s = new String(ch, start, length);
			if (name_bool) {
				atnum = new Tnum();
				setTnum(s, atnum);
			} else if (data_bool) {
				data.append(s);
			} else if (entry_bool) {
				if (id_bool) {
					key = new Integer(s);
				} else if (f_bool) {
					forme = s;
				} else if (freq_bool) {
					mapFreq.put(key, new Integer(s));
				}
			}
		}
		public void startElement(String uri, String localName, String qName, Attributes attributes) {
			if (uri.equals(Textometre.TEXTOMETRE_NAMESPACE)) {
				if (localName.equals("dico")) {
					tnum_bool = true;
					//quand on commence un tnum on initialise son dictionnaire
					mapFreq =  new HashMap<Integer, Integer>();
					properties = new HashMap<Object, Integer>();
					invert =  new HashMap<Integer, Object>();
				} else if (localName.equals("name")) {
					name_bool = true;
				} else if (localName.equals("entry")) {
					entry_bool = true;
				} else if (localName.equals("data")) {
					data_bool = true;
				} else if (localName.equals("f")) {
					f_bool = true;
				} else if (localName.equals("freq")) {
					freq_bool = true;
				} else if (localName.equals("id")) {
					id_bool = true;
				}
			}
		}
		public void endElement(String uri, String localName, String qName) {
			if (uri.equals(Textometre.TEXTOMETRE_NAMESPACE)) {
				if (localName.equals("dico")) {
					tnum_bool = false;
					//quand on a fini un tnum, on crit son dictionnaire
					atnum.setDico(invert, mapFreq, properties);
				} else if (localName.equals("name")) {
					name_bool = false;
				} else if (localName.equals("entry")) {
					entry_bool = false;
				} else if (localName.equals("data")) {
					data_bool = false;
				} else if (localName.equals("f")) {
					if ((forme == null) || (forme.equals(""))) {
						forme = "\n";
					}
					invert.put(key, forme);
					properties.put(forme, key);
					forme = "";
					f_bool = false;
				} else if (localName.equals("freq")) {
					freq_bool = false;
				} else if (localName.equals("id")) {
					id_bool = false;
				} else if (localName.equals("trame")) {
					read_data(data.toString());
				}
			}
		}
	}
}